# Face fitting with collapsed nodes and version mapping.
# Created 23/1/2005 by Glenn Ramsey
# Based on example 21h.

set dir example 21l
# Set up a path to use so that the Perl stuff can find the files. I expected
# the above to do this but it doesn't.
$epath = "$ENV{CMISS_EXAMPLES}/2/21/21l/cmiss_input/";

use lib "$ENV{CMISS_ROOT}/cmiss_perl/lib";

use strict;
use warnings;

use CmUtils::File::Ipnode qw( readIpnode);
use CmUtils::File::Ipfiel qw( writeIpfiel );
use CmUtils::File::Ipelem qw( readIpelem );
use CmUtils::File::Ipelfd qw( writeIpelfd );

$name = 'collapse';
$tot_itt=2;
$off=1000;
$offi=100;

# To use mapping; in the ippara file USE_MAPS needs to be set to 1
# and NYYM may need to be increased depending on the problem. 
fem def para;r;${name};example          # Declares array dimensions

fem def coord;r;${name};example         # Defines the coordinate system
# substitute the above line with the one below and see what happens
#fem def coord;r;${name}_nomap;example

fem def base;r;${name}_cubic;example    # Defines tri-cubic Hermite basis funcs.
fem def node;r;${name}_version;example  # Reads nodal information
fem def elem;r;${name}_version;example  # Read element information

fem update nodes derivative 1 linear;   # Updates the derivatives in all three
fem update nodes derivative 2 linear;   # xi directions
fem update nodes derivative 3 linear;   #
fem update scale_factor normalise;

# export the initial mesh for viewing
fem export node;${name}_initial as ${name}_initial offset $offi;
fem export elem;${name}_initial as ${name}_initial offset_node $offi offset_elem $offi;

# In order for the fitting to work properly you must define some fields
# that have exactly the same topography as the mesh. There are some
# Perl utilities which can help with that, otherwise you'll have to do
# it by hand.

# # Use the Perl utilities to create a field that matches the node geometry.
# # First write out the updated nodes. 
# fem def nodes;w;${name}_version_auto;example 
# # then convert to a field
# @ngr = readIpnode($epath.${name}."_version_auto");
# $options->{fields} = "coordinates";
# $options->{noVersions} = 0;
# $options->{noDerivatives} = 0;
# writeIpfiel($epath.${name}."_version_auto", @ngr, $options);
# # read in the field we just created
# fem def field;r;${name}."_version_auto";example

# If no field already exists then this command creates one matching
# the geometry. If a field that doesn't match the geometry already
# exists then its values are updated but the field is not changed (gotcha!)
fem update field from geometry

# Create a field that matches the elements.
# As of 21/1/2005 the default for "fem define elements;d field" is to copy
# the geometry elements so the code below is not required, but it is left
# here for reference.
# @egr = readIpelem(${name}."_version", @ngr);
# $options->{versions} = 1;
# writeIpelfd(${name}."_version_auto", \@egr, $options);
#fem def elem;r;${name}_version_auto;example field

fem define elements;d field # create an element field matching the geometry 

# Write out the fields so we can check them manually.
fem def field;w;${name}_created_field;example  
fem def elem;w;${name}_created_field;example field

# The set of faces which will be fitted to the data needs to be defined.
# There are a number of filters which can be used such as XiN high/low,
# external and element groups. Also face numbers can be specified but 
# that is tedious and might need to be redone if the mesh changes.
# E.g. fem group faces all Xi3 high elements MY_ELEM_GROUP
# See fem group faces ? for complete list of options.

# Here we use the external faces which is the same as all in this case.
fem group faces allfaces external as EXT_FACES   # Defines face group of external faces

fem def data;r;${name};example              # Reads in data to fit to

fem def xi;c closest_face faces EXT_FACES   # Projects data onto faces
                                            # only in group EXT_FACES
                                 
fem def xi;w;${name} closest_face faces EXT_FACES  # Writes out xi positions
fem li data error;

fem export data;${name} as ${name}

for ($fit_itt=1; $fit_itt<=$tot_itt; $fit_itt++)
  {
   fem def fit;r;${name}_${fit_itt};example geometry faces EXT_FACES
   # Try commenting out the following line and see what happens to the mesh.
   fem define mapping;r;${name};example
   fem fit
   fem update node fit
   fem update scale_factor normalise
   fem def xi;c closest_face faces EXT_FACES old

   system "echo ' ======================================================' "    
   fem li data error
   system "echo ' ITERATION ${fit_itt} DONE' "
   system "echo ' ======================================================' "
  }
fem export nodes;${name}_fitted as ${name}_fitted offset $off;
fem export elem;${name}_fitted as ${name}_fitted offset_node $off offset_elem $off;
fem quit
