#Example 513: Membrane deformation using the Tong & Fung skin strain energy function

#Deformation of a membrane, using the strain energy equation
#for skin detailed in Tong & Fung.

#See:
#  "The Stress-Strain Relationship for the Skin"
#  P. Tong and Y-C. Fung.
#  J. Biomechanics, Vol 9, 1976.
#  pp 649-657

#Define a coordinate system. The problem details a two-dimensional
#membrane in 3d space.

fem de coor 3,1

#Define four nodes describing the initial 
#membrane geometry (a square).

fem de node;r;skin;example

#Define a bilinear basis function.

fem de base;r;;example

#Define the membrane as one element and assign fibre
#information to that element. The fibres are aligned
#parallel to Xi(1).

fem de elem;r;;example            #1 element
fem de fibr;r;;example           
fem de elem;r;;example fibre

#Define the problem equations.
# Finite elasticity.
# Membrane theory.
# Incompressible.

fem de equa;r;;example            

#Define the material.
#This is default for the first three questions.
#The strain enegry question W is an exponential equation.
#The Tong & Fung skin function is chosen.
#This function has 13 constitutive law parameters which are
#taken from the paper by Tong and Fung.
#Membrane thickness is taken to be 1.
#Gravity is not used and so is set to 0.


fem de mate;r;;example          

#Define the initial and boundary conditions.
#No pressure is applied on the Xi(3) faces of the membrane.
#Initial displacements are all zero.
#In the Xi(1) direction:
# Nodes 1 and 3 are tethered with zero displacement.
# Nodes 2 and 4 are displaced by 0.1 (giving an extension 
#               ratio of 1.1.
#In the Xi(2) direction:
# Nodes 1 and 2 are tethered with zero displacement.
# Nodes 3 and 4 are allowed to move freely.
#There are no constraints on displacement in Xi(3).
#There are no force boundary conditions.

fem de init;r;;example

#Define the solution method.
        
fem de solv;r;;example

fem solve

#List the solution values

fem list node sol
fem list strain
fem list stress

#Loop over the commands below 20 times. The solves the problem
#with an incremental increase of 50% of the initial displacement
#finishing with an overall extension ration of 2.15.

for $M (0..20)
{
  fem solve increment 0.5 error 1.0E-10
  fem list node sol
  fem list strain
  fem list stress
}

quit


















