#This example uses a full forvard solve method to estimate material parameters from a in-vivo multiaxial loading experiment on human skin. A 
#loading cycle including a total of ten deformed states are used to estimate five parameter values in the Tong and Fung skin function. A testing 
#version of this problem is found in example 7183 

# If the example path is not set, default to current directory
if (!defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}
#**************************************************************
set num_threads 4
@DATASETS = (4,5,6,8,9,10,11,12,13,14)						#Deformed states used in the parameter estimation

fem define parameters;r;multiaxes;example
	
fem define bases;r;bases_cube;example						#Bi-cubic hermite for geomety, bi-linear for fibre 
fem define nodes;r;coordinates_cube_undeformed;example
fem define elements;r;elements_cube;example					#Circular mesh with 192 elements

fem define fibre;r;fibre;example
fem define elements;r;fibreelem_cube;example fibre

fem define equation;r;equation;example						#Incompressible membrane (uses plain-stress)
fem define material;r;tongfung;example						#Tong and Fungs skin function
	
fem group nodes 1..32 as BOUNDARY
	
fem define data;r;datapoints_undeformed;example					#Reads the undeformed coordinates of the datapoints
foreach $index (@DATASETS[1..$#DATASETS]){
	fem define data;r;datapoints_undeformed;example append
}

fem define xi;c closest								#Defines the xi positions of the datapoints

fem define data;r;datapoints_${DATASETS[0]};example as IMAGE_${DATASETS[0]}	#Reads the deformed coordinates of the datapoints
foreach $index (@DATASETS[1..$#DATASETS]){
	fem define data;r;datapoints_${index};example append as IMAGE_${index}
}


fem define initial;r;initskinsolve_cube;example					#Defines a force boundary problem
fem define solve;r;solve;example
fem define optimise;r;tongfung_five_cube;example				#Estimate five material parameters
	
fem define nodes;r;displacements_cube_$DATASETS[-1];example dep 2 nc 1			
fem define nodes;r;force_cube_$DATASETS[-1];example dep 2 nc 2			#Reads the experimentaly applied forces
fem define nodes;r;force_cube_residual;example dep 1 nc 2			#Reads the residual forces estimated in example 7181

fem solve increment 0.01 iterate 100 error 0.00000001
fem solve increment 0.09 iterate 100 error 0.00000001
for $M (1..9)
{
	fem solve increment 0.1 iterate 100 error 0.00000001			#Solves the force boundary problem in 11 increments
}

optimise									#The optimiser calles the eval_resid_cube.com file

fem list data errors deformed

