
# This version of estimate.pl was adjusted to estimate passive material parameters 
# in a heart model. 
 
# Strains (stretches) from an inflated heart model is used as measurements.
# First the model is inflated with the "true" parameters and 
# a set of syntetic data is extracted as our "measurements"
#
# The estimation routine is then started where the estimated parameters
# are initiallly guessed to be 20% higher than the true parameters.
#
# The parameters are then estimated using a Gauss-Newton method

# The result file is log file that keeps track of what was
# done for this estimation trial and the intermediate and end results
if (-e "result.txt")
{
  system("rm result.txt");
}
CORE::open(RESULTFID, ">> result.txt") || die "Couldn't open result file";

# Make sure the PDL can be found in your path
use PDL;
use PDL::Slatec;
use File::Copy "cp";

#remember to check which materialfile to input!!!
cp("ellipsoid.ipmate", "editedEllipsoid.ipmate");
$cm = "/product/cmiss//cm/bin/mips-irix/cm-mt";
# There are 3 ipmate files:
# 1. orgellipsoid.ipmate contains all the true parameters
# 2. ellipsoid.ipmate is the same as orgellipsoid, but the parameters
# that are not estimated may be chosen to be set to other values
# than their true parameter values.
# 3. editedEllipsoid.ipmate is the file used by the cmiss-script and
# contains the currently used parameters for the currrent simulation


# Copy original and differences in parameters into result.txt
CORE::open(INFID, "orgellipsoid.ipmate") || die "Couldn't open orgellipsoid.ipmate";
while (<INFID>)
{
  print RESULTFID;
}
CORE::close(INFID);
print RESULTFID "\n\nDifferences:  diff orgellipsoid.ipmate ellipsoid.ipmate\n";
system("diff orgellipsoid.ipmate ellipsoid.ipmate >> result.txt");
print RESULTFID "\n Diff finished\nInit file\n\n\n";
CORE::open(INFID, "Base_node_restricted.ipinit") || die "Couldn't open Base_node_restricted.ipinit";
while (<INFID>)
{
  print RESULTFID;
}
CORE::close(INFID);
print RESULTFID "\nListing of Base_node_restricted.ipinit finished\n\n\n";
# Copy estimate.pl into result.txt
print RESULTFID "\n\nestimate.pl:\n\n";
CORE::open(INFID, "estimate.pl") || die "Couldn't open estimate.pl";
while (<INFID>)
{
  print RESULTFID;
}
CORE::close(INFID);
print RESULTFID "\nListing of estimate.p finished\n\n\n";
print RESULTFID "\n\nEstimations:\n";


# Simulate the model with the true parameters and obtain the
# measurements. May comment out these two lines if this has been
# done before.
system("$cm ellipsoid1");
cp("heartStrain.opstra", "defaultHeartStrain.opstra");
$y0 = extractStrains("defaultHeartStrain.opstra");
$nDataPoints = $y0->getdim(1);
if(1)
{
  print RESULTFID "y0 without noise:", $y0;
  print RESULTFID "\n******************************************************\n";
  #$meanStrain = avg(abs($y0-ones(1,$nDataPoints)));
  #$y0 = addNoise($y0, 1/2.5);

  # You may want to add noise to the syntetic data.
  # See createNoise.pl for an example of how to create noise.
  $y0 = $y0 + addNoiseFromFile("randomNoise000.txt", $nDataPoints);
}
print RESULTFID "y0:", $y0;
print RESULTFID "\n******************************************************\n";

# Initial guess of the parameter values
#$alpha = pdl [[2], [0.523], [1.351], [2], [0.681], [1.351], [2], [1.037], [0.398]];
$alpha = pdl [[0.523], [0.681], [1.037]];
$alpha = 1.2*$alpha;
# This variable controls which parameters that will be estimated and changed in the parameter
# file (function editMaterialFile($alpha)):
#$boolEstParam = pdl [[1], [1], [1], [1], [1], [1], [1], [1], [1]];
$boolEstParam = pdl [[0], [1], [0], [0], [1], [0], [0], [1], [0]];


############ GaussNewton parameter estimation routine  #################
# Note that a whole lot of information to analyze observability etc
# is extracted too, not only what is necessary to estimate the parameters

#print "alpha: $alpha\n\n";
#$nParameters = $alpha->getdim(1)-1;
$nParameters = $alpha->getdim(1);
$return = editMaterialFile($alpha);
system("$cm ellipsoid1");
$y1 = extractStrains("heartStrain.opstra");
#$y1 = pdl $y0;
print RESULTFID "y1:\n", $y1;
($y1_1, $y1_2, $y1_3) = seperateComponents($y1);

#print "y1: $y1\n";
print RESULTFID "alpha: $alpha\n\n";
print RESULTFID "\n******************************************************\n";

$error = $y1-$y0;
print(sprintf("Abs val of error: %0.19f\n", normOfVector($error)));
print RESULTFID "Error: $error\n";
print RESULTFID (sprintf("Abs val of error: %0.19f\n", normOfVector($error)));
print RESULTFID (sprintf("Mean error: %0.19f\n\n\n", avg($error)));

#die "Test finished.";

$iterations = 1;
$error = ones(1,$nDataPoints)*9999;
while (($iterations < 7)  && (normOfVector($error) > 0.001))
{ # iterations
  # The finishing criteria may be changed to check the
  # change in parameter increments rather than the error

  $S = zeroes($nParameters,$nDataPoints);
  $S1 = zeroes($nParameters,$nDataPoints/3);
  $S2 = zeroes($nParameters,$nDataPoints/3);
  $S3 = zeroes($nParameters,$nDataPoints/3);
  #$y2 = zeroes(1,$nDataPoints);
  for ($n = 0; $n < $nParameters; $n++)
  {
    $alpha2 = pdl $alpha;

    $incrFactor = 1.1;  
    PDL::Core::set $alpha2,0,$n,(at($alpha,0,$n)*$incrFactor);
    #PDL::Core::set $alpha2,0,$a3Index,(at($alpha2,0,$a1Index)*$poleFactor);
    print sprintf("Changing parameter #: %d\n", $n+1);
    print "alpha: $alpha2\n";
    print RESULTFID sprintf("Changing parameter #: %d\n", $n+1);
    print RESULTFID "alpha: $alpha2\n";
    print RESULTFID "\n******************************************************\n";

    $return = editMaterialFile($alpha2);
    system("$cm ellipsoid1");
    cp("deformed.ipnode", "deformed0$n.ipnode");
    #cp("deformed0$n.ipnode", "deformed.ipnode"); # For testing
    $y2 = extractStrains("heartStrain.opstra");
    print RESULTFID "y2:", $y2;
    #print "y2: $y2\n";
    #print $y2-$y1;
    #PDL::Core::set $S,$n,:,($y2-$y1);
    $tmp = $S->slice("$n,:"); 
    $tmp .= ($y2-$y1)/(at($alpha2,0,$n) - at($alpha,0,$n));
    #print $S, "\n";
    print RESULTFID (sprintf("\nNorm of y2-y1: %0.19f\n", normOfVector($y2-$y1)));
    print RESULTFID "\n******************************************************\n";
    ($y2_1, $y2_2, $y2_3) = seperateComponents($y2);

    $tmp = $S1->slice("$n,:"); 
    $tmp .= ($y2_1-$y1_1)/(at($alpha2,0,$n) - at($alpha,0,$n));
    print RESULTFID (sprintf("\nNorm of y2_1-y1_1: %0.19f\n", normOfVector($y2_1-$y1_1)));
    $tmp = $S2->slice("$n,:"); 
    $tmp .= ($y2_2-$y1_2)/(at($alpha2,0,$n) - at($alpha,0,$n));
    print RESULTFID (sprintf("\nNorm of y2_2-y1_2: %0.19f\n", normOfVector($y2_2-$y1_2)));
    $tmp = $S3->slice("$n,:"); 
    $tmp .= ($y2_3-$y1_3)/(at($alpha2,0,$n) - at($alpha,0,$n));
    print RESULTFID (sprintf("\nNorm of y2_3-y1_3: %0.19f\n", normOfVector($y2_3-$y1_3)));

    cp("heartStrain.opstra", "heartStrain.$n.opstra");
  }


  # Calculate sensitivity matrix, find rank and eigenvalues
  # Calculate new parameters
  # Check if converged - break loop

  $s = zeroes($S->getdim(0),$S->getdim(1));
  $s1 = zeroes($S1->getdim(0),$S1->getdim(1));
  $s2 = zeroes($S2->getdim(0),$S2->getdim(1));
  $s3 = zeroes($S3->getdim(0),$S3->getdim(1));
  for ($n = 0; $n < $nParameters; $n++)
  {
    $tmp = $s->slice("$n,:"); 
    $tmp .= $S->slice("$n,:")/normOfVector($S->slice("$n,:"));
    #$tmp .= transpose PDL::Primitive::norm(transpose $S->slice("$n,:"));

    $tmp = $s1->slice("$n,:"); 
    $tmp .= $S1->slice("$n,:")/normOfVector($S1->slice("$n,:"));
    $tmp = $s2->slice("$n,:"); 
    $tmp .= $S2->slice("$n,:")/normOfVector($S2->slice("$n,:"));
    $tmp = $s3->slice("$n,:"); 
    $tmp .= $S3->slice("$n,:")/normOfVector($S3->slice("$n,:"));
  }
  $ss = (transpose $s) x $s;
  $SS = (transpose $S) x $S;
  print RESULTFID "\n******************************************************\n";
  print RESULTFID "Iteration: ", $iterations, "\n";
  print RESULTFID "Sensitivity matrix:", $SS;

  $ss1 = (transpose $s1) x $s1;
  $SS1 = (transpose $S1) x $S1;
  $ss2 = (transpose $s2) x $s2;
  $SS2 = (transpose $S2) x $S2;
  $ss3 = (transpose $s3) x $s3;
  $SS3 = (transpose $S3) x $S3;
  $return = writeMatrix("ss.xls",$ss);
  $return = writeMatrix("ss1.xls",$ss1);
  $return = writeMatrix("ss2.xls",$ss2);
  $return = writeMatrix("ss3.xls",$ss3);
  $return = writeMatrix("SS.xls",$SS);
  $return = writeMatrix("SS1.xls",$SS1);
  $return = writeMatrix("SS2.xls",$SS2);
  $return = writeMatrix("SS3.xls",$SS3);

  # Calculate the sensitivity of the model response to a change in parameter.
  # I.e. find the change of the model response when a parameter is varied.
  # This is done by looking at the diagonal elements on the SS matrix, but
  # we have to compensate because in SS we have divided by delta-parameter
  # when we calculated S. Thus we have to multybly by the square of delta-parameter.
  $sensValues = zeroes($nParameters,1);
  $sensValues1 = zeroes($nParameters,1);
  $sensValues2 = zeroes($nParameters,1);
  $sensValues3 = zeroes($nParameters,1);
  for ($n = 0; $n < $nParameters; $n++)
  {
    PDL::Core::set $sensValues,$n,0,(sqrt (at($SS,$n,$n)*((at($alpha,0,$n)*($incrFactor-1))**2)));
    PDL::Core::set $sensValues1,$n,0,(sqrt (at($SS1,$n,$n)*((at($alpha,0,$n)*($incrFactor-1))**2)));
    PDL::Core::set $sensValues2,$n,0,(sqrt (at($SS2,$n,$n)*((at($alpha,0,$n)*($incrFactor-1))**2)));
    PDL::Core::set $sensValues3,$n,0,(sqrt (at($SS3,$n,$n)*((at($alpha,0,$n)*($incrFactor-1))**2)));
  }
  print RESULTFID "Sensitivity valus:", $sensValues;
  $return=writeSensValues("sensValues.xls",$sensValues);
  $return=writeSensValues("sensValues1.xls",$sensValues1);
  $return=writeSensValues("sensValues2.xls",$sensValues2);
  $return=writeSensValues("sensValues3.xls",$sensValues3);


  print RESULTFID "Normalised sensitivity matrix:", $ss;
  print "Sensitivity matrix:\n", $SS;
  print "Normalised sensitivity matrix:\n", $ss;
  ($eigvecs, $eigvals) = PDL::Math::eigens($ss);
  #($eigvals, $eigvecs) = eigsys($ss);
  print RESULTFID "Eigenvalues:\n", $eigvals, "\n";
  print "Eigenvalues:\n", $eigvals, "\n";
  if (min(abs($eigvals)) < 1e-6)
  {
    print "\nWarning: Eigenvalue(s) of sensitivity matrix less than 1e-6!\n";
    print sprintf("Minimum eigenvalue: %f\n\n", min(abs($eigvals)));
  }
  if (min(abs($eigvals)) eq 0)
  {
    print "\nWarning: Eigenvalue(s) of sensitivity matrix equals zero\n";
    print "Rank of matrix is less than number of parameters. Exiting!\n\n";
    die;
  }
  $const = zeroes(float, $nParameters, $nParameters); # make everything zero
  ($tmp = $const->diagonal(0,1)) .= 0.001;
  #print RESULTFID "Test av const matrix:\n", $const;
  #$SS = (transpose $S) x $S + $const;
  $SS = (transpose $S) x $S;
  print $SS;
  $error = $y0-$y1;
  $h = matinv($SS) x (transpose $S) x $error;
  #$h = matinv($SS) x (transpose $S) x ($y0-$y1);
  print "Increment h:\n", $h;


  # Write to result-file
  #CORE::open(RESULTFID, ">> result.txt") || die "Couldn't open result file";
  print RESULTFID "Old alpha: ", $alpha;
  print "Old alpha: ", $alpha, "\n";

  $alpha2 = pdl $alpha; #Need this variable in case of new negative alpha values
  $tmp = $alpha; 
  $tmp .= $alpha + $h;
  #PDL::Core::set $alpha,0,$a3Index,(at($alpha,0,$a1Index)*$poleFactor);
  print "New alpha: ", $alpha, "\n";

  print RESULTFID "Increment h:", $h;
  print RESULTFID "New alpha: ", $alpha;

  # Check to see if parameteres are within physiological reasonable levels:
  $boolAlphaChanged = 0;
  ($alpha, $boolAlphaChanged) = checkAlpha($alpha, $alpha2, $boolAlphaChanged);
  if ($boolAlphaChanged)
  {
    print RESULTFID "Negative value or value outside accepted range changed!\nNew alpha: ", $alpha;
    $boolAlphaChanged = 0;
  }


  # Find new reference y1 and new error with new alpha
  $return = editMaterialFile($alpha);
  system("$cm ellipsoid1");
  cp("deformed.ipnode", "deformed111.ipnode");
  $y1 = extractStrains("heartStrain.opstra");
  print RESULTFID "y1:", $y1;
  #print "y1: $y1\n";
  $error = $y1-$y0;
  #$error = ($y1-$y0);
  #print "Error: $error\n";
  print(sprintf("Abs val of error: %0.19f\n", normOfVector($error)));
  print(sprintf("Mean error: %0.19f\n", avg($error)));

  print RESULTFID "Error: $error\n";
  print RESULTFID (sprintf("Abs val of error: %0.19f\n", normOfVector($error)));
  print RESULTFID (sprintf("Mean error: %0.19f\n\n\n", avg($error)));
  #CORE::close(RESULTFID);

  $iterations++;
} #iterations

CORE::close(RESULTFID);



###################### Subroutines ###############################################



sub editMaterialFile
{
  my ($alpha) = @_;
  my $n;
  my $line;
  my $paramIndex = 0;

  CORE::open(INFID, "ellipsoid.ipmate") || die "Couldn't open ellipsoid.ipmate";
  CORE::open(OUTFID, ">editedEllipsoid.ipmate") || die "Couldn't open editedEllipsoid.ipmate";
  while (<INFID>)
  {
    print OUTFID;
    if((/ Specify whether the fibre axis coefficient/) && at($boolEstParam,0,0))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.194E+01]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the fibre axis pole is/) && at($boolEstParam,0,1))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the fibre axis curvature is/) && at($boolEstParam,0,2))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet axis coefficient/) && at($boolEstParam,0,3))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet axis pole is/) && at($boolEstParam,0,4))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet axis curvature is/) && at($boolEstParam,0,5))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet-normal axis coefficient/) && at($boolEstParam,0,6))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet-normal axis pole is/) && at($boolEstParam,0,7))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet-normal axis curvature is/) && at($boolEstParam,0,8))
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the fibre angle std dev/) && 0)
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet angle std dev/) && 0)
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet-normal std dev 1/) && 0)
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
    if((/ Specify whether the sheet-normal std dev 2/) && 0)
    {
      for ($n = 1; $n <= 6; $n++)
      {
        $line = <INFID>;
        print OUTFID $line;
      }
      $line = sprintf(" The value is [0.523E+00]:  %0.7f\n", at($alpha,0,$paramIndex));
      $paramIndex++;
      print OUTFID $line;
      $line = <INFID>;
    }
  }
  CORE::close(INFID);
  CORE::close(OUTFID);
  return(1);
}




# Subroutine seperateComponents
# This function takes the strain-vector containing all three main
# strain components for each Gauss point and seperate that vector
# into three vectors containing the three seperate strain components
# in each vector.
sub seperateComponents
{
  my ($vec) = @_;
  my $vec1;
  my $vec2;
  my $vec3;
  my $length;
  my $n;
  my $k=0;
  my $p;


  $length = $vec->getdim(1);
  $vec1 = zeroes(1,$length/3);
  $vec2 = zeroes(1,$length/3);
  $vec3 = zeroes(1,$length/3);

  $n=0;
  while($n < $length)
  {
    $p=$n;
    PDL::Core::set $vec1,0,$k,at($vec,0,$p);
    $p++;
    PDL::Core::set $vec2,0,$k,at($vec,0,$p);
    $p++;
    PDL::Core::set $vec3,0,$k,at($vec,0,$p);
    $n=$n+3;
    $k++;
  }
  return($vec1,$vec2,$vec3);
}


# Subroutone writeMatrix writes out sensitivity matrix to a format easily 
# imported in Excel
sub writeMatrix
{
  my ($filename, $mat) = @_;
  my $c;
  my $r;

  CORE::open(FID, ">$filename") || die "Couldn't open $filename";
  for ($r=0; $r<$mat->getdim(1); $r++)
  {
    for ($c=0; $c<$mat->getdim(0); $c++)
    {
      print FID sprintf("\t%0.8f", at($mat,$c,$r));
    } 
    print FID "\n";
  }
#  print FID $mat;
  CORE::close(FID);
  return(1);
}


# Subroutone writeSensValues writes out sensitivity values to a format easily 
# imported in Excel
sub writeSensValues
{
  my ($filename, $vec1) = @_;
  my $c;

  CORE::open(FID, ">$filename") || die "Couldn't open $filename";
  for ($c=0; $c<$vec1->getdim(0); $c++)
  {
    print FID sprintf("\t%0.8f", at($vec1,$c,0));
  } 
  print FID "\n";
#  print FID $mat;
  CORE::close(FID);
  return(1);
}


# Subroutone extractStrains 
# reads in the strain components for each gauss point 
# iin a .opstra file and puts them into a vector.
sub extractStrains
{
  my ($filename) = @_;
  my $strain = zeroes(1,10000);
  my $value;
  my $index = 0;

  CORE::open(IN_FID, $filename) || die "Couldn't open $filename";  

  while ($line = <IN_FID>)
  {
    if(grep(/       Physical Greens strains/, $line))
    {
      $line = <IN_FID>; 
      ($value) = ($line =~ m/       EG\(1,j\):.+EXR\(1\):(.+)RI1:.+/);
      $value =~ s/D/e/;
      PDL::Core::set $strain,0,$index,$value;
      $index++;
      $line = <IN_FID>; 
      ($value) = ($line =~ m/          2.+\s2(.+)RI2:.+/);
      $value =~ s/D/e/;
      PDL::Core::set $strain,0,$index,$value;
      $index++;
      $line = <IN_FID>; 
      ($value) = ($line =~ m/          3.+\s3(.+)RI3:.+/);
      $value =~ s/D/e/;
      PDL::Core::set $strain,0,$index,$value;
      $index++;
    }    
  }
  CORE::close(IN_FID);
  $index--;
  return($strain->slice("0,0:$index"));
}


# Subroutine checkAlpha
# This routine checks if any og the new calculated parameters are either
# negative or outside a accepted range for the parameter. If a parameter is
# negative, it is set to a fraction (specified in code) of its old value ($alpha2).
# If a parameter is outside the accepted range it is set to the limit value.
sub checkAlpha
{
  my ($alpha, $alpha2, $boolAlphaChanged) = @_;
  my $n = 0;

  my $a1Index = 0;
  my $a1Min   = 0.1;
  my $a1Max   = 0.8;

  my $a2Index = 1;
  my $a2Min   = 0.1;
  my $a2Max   = 1.2;

  my $a3Index = 2;
  my $a3Min   = 0.2;
  my $a3Max   = 5;

  my $b1Index = 2;
  my $b1Min   = 0.1;
  my $b1Max   = 3;
 
  my $b2Index = 5;
  my $b2Min   = 0.1;
  my $b2Max   = 20;

  #Check for negative values:
  for ($n = 0; $n < $alpha->getdim(1); $n++)
  {
    if (at($alpha,0,$n)<=0)
    {
      PDL::Core::set $alpha,0,$n,(at($alpha2,0,$n)*0.5);
      print RESULTFID sprintf("Negative parameter #: %d Changed to: %f\n", $n+1, at($alpha,0,$n));
      $boolAlphaChanged = 1;
    }
  }

  if (0)
  {
    ($alpha, $boolAlphaChanged) = checkParameter($alpha, $alpha2, $boolAlphaChanged, $a1Index, $a1Max, $a1Min, "Fiber pole");
  }
  if (0)
  {
    ($alpha, $boolAlphaChanged) = checkParameter($alpha, $alpha2, $boolAlphaChanged, $a2Index, $a2Max, $a2Min, "Sheet pole");
  }
  if (0)
  {
    ($alpha, $boolAlphaChanged) = checkParameter($alpha, $alpha2, $boolAlphaChanged, $a3Index, $a3Max, $a3Min, "Sheet-normal pole");
  }
  if (0)
  {
    ($alpha, $boolAlphaChanged) = checkParameter($alpha, $alpha2, $boolAlphaChanged, $b1Index, $b1Max, $b1Min, "Fiber curvature");
  }
  if (0)
  {
    ($alpha, $boolAlphaChanged) = checkParameter($alpha, $alpha2, $boolAlphaChanged, $b2Index, $b2Max, $b2Min, "Sheet curvature");
  }
    
  return($alpha, $boolAlphaChanged);
}


# Subroutine checkParameter
# This routine checks if one parameters is outside the specified range and in
# if a parameter is outside the accepted range it is set to the limit value.
sub checkParameter
{
  my ($alpha, $alpha2, $boolAlphaChanged, $index, $maxLim, $minLim, $name) = @_;

  if ((at($alpha,0,$index)>$maxLim) && 1)
  {
    PDL::Core::set $alpha,0,$index,$maxLim;
    print RESULTFID sprintf("$name changed to: %f\n", at($alpha,0,$index));
    $boolAlphaChanged = 1;
  }
  if ((at($alpha,0,$index)<$minLim) && 1)
  {
    PDL::Core::set $alpha,0,$index,$minLim;
    print RESULTFID sprintf("$name changed to: %f\n", at($alpha,0,$index));
    $boolAlphaChanged = 1;
  }
  
  return ($alpha, $boolAlphaChanged);
}

# Subroutine addNoise
# This routine adds random noise to a column vector.
# The noise level is set by multFac.
sub addNoise
{
  my ($yIn, $multFac) = @_;
  my $nPoints = $yIn->getdim(1);
  my $noise = zeroes(1,$nPoints);  
  my $n;
    
  srand;
  for ($n = 0; $n < $nPoints; $n++)
  {
    PDL::Core::set $noise,0,$n,(((rand)-0.5)*$multFac);
  }
  print RESULTFID "\nNoise:", $noise;
  print RESULTFID "\n******************************************************\n";
  # For testing:
  $nPoints--; #pga slice kommando m indeksen settes ned.
  #$meanNoise = avg($noise->slice("0,0:$nPoints"));
  my $meanNoise = avg(abs($noise->slice("0,0:$nPoints")));
  
  print RESULTFID "\nMean noise: $meanNoise\n";
  my $stdv = std($noise->slice("0,0:$nPoints"));
  print RESULTFID "Standard dev.: $stdv\n";
  print RESULTFID "Minimum noise: ", min($noise);
  print RESULTFID "\nMaximum noise: ", max($noise);
  print RESULTFID "\n******************************************************\n";
  $nPoints++;
  return ($yIn+$noise);
}


# Subroutine addNoiseFromFile
# This routine reads in noise from a specified file and returns it as 
# a column vector.
sub addNoiseFromFile
{
  my ($filename, $nPoints) = @_;
  my $noise = zeroes(1,$nPoints);  
  my $value;
  my $n=0;
  my $line;

  CORE::open(IN_FID, $filename) || die "Couldn't open $filename";  
  while ($line = <IN_FID>)
  {
    if ($line =~ m/\[(.+)\]/)
    {
      #print RESULTFID "\n$n   ", $1;
      if ($n >= $nPoints)
      {
        print RESULTFID "\nError: Too many noise elements compared to measurements!\n";
        die "\n\n\nError: Too many noise elements compared to measurements!\n"; 
      }
      PDL::Core::set $noise,0,$n,$1;
      $n++;
    }    
  }
  CORE::close(IN_FID);
  print RESULTFID "\nNoise:", $noise;
  print RESULTFID "\n******************************************************\n";
  # For testing:
  $nPoints--; #pga slice kommando m indeksen settes ned.
  my $meanNoise = avg(abs($noise->slice("0,0:$nPoints")));
  print RESULTFID "\nMean noise: $meanNoise\n";
  my $stdv = std($noise->slice("0,0:$nPoints"));
  print RESULTFID "Standard dev.: $stdv\n";
  print RESULTFID "Minimum noise: ", min($noise);
  print RESULTFID "\nMaximum noise: ", max($noise);
  print RESULTFID "\n******************************************************\n";
  $nPoints++;
  return ($noise);
}


sub normOfVector
{
  my ($vector) = @_;
  my $n;
  my $result = 0;
  
  for ($n = 0; $n < $vector->getdim(1); $n++)
  {
    $result += at($vector,0,$n)**2;
  }
  return(sqrt $result);  
}


# This routine calculates the standard deviation of a coumn vector
sub std
{
  my ($vec) = @_;

  my $stdv;
  my $nDataPoints;
  my $n;
  my $sum = 0;

  $nDataPoints = $vec->getdim(1);
  $n = $nDataPoints-1;
  $mean = avg($vec->slice("0,0:$n"));
  for($n=0; $n<$nDataPoints; $n++)
  {
    $sum = $sum + (at($vec,0,$n)-$mean)**2;
  }
  $stdv = sqrt($sum/($nDataPoints-1));
  
  return ($stdv);  
}


### Old stuff not used for the strain case
# These were used when the measurement points on the endocardial surface
# was used as measurements instead of strains.

# Subroutine readResult:
# Read in datapoints from dataPositions.opdata: get point number and the error 
# between that point and the surface. This file may have fewer datapoints than 
# the surface file. Sort this list in ascending datapoint order.
# Calculate the mean error. For each datapoint where error is over a threshold
# -> set badDataPoint.
# Read in the surfacepoints from surfacePositions.ipdata: the datapoint number 
# and its lambda value. If lamda equals 0, that means no projection was obtained
# -> set badDataPoint.
# The output vector is the lambda-values of on the surface.
sub readResult
{
  my ($badDataPoints) = @_;
  my $line;
  my $dummy;
  my $dummyMatrix1 = zeroes(2,$badDataPoints->getdim(1)); #dataPoint, error
  my $filename = "dataPositions.opdata";
  my $n1 = 0;
  my $n = 0;
  
  CORE::open(FID, $filename) || die "Couldn't open $filename file";
  $line = <FID>;
  while ($line = <FID>) 
  {
    if ($line =~ m/(\d+)\) nd=\s*(\d+) NDP=.+LD=.+ERR=(.+)/)
    {
      #print "$1 $2 $3\n";
      $dummy = $2;
      $dummy =~ s/d/e/i;      
      PDL::Core::set $dummyMatrix1,0,$n1,$dummy;
      $dummy = $3;
      $dummy =~ s/d/e/i;      
      PDL::Core::set $dummyMatrix1,1,$n1,$dummy;

      # Some old code for reading in the lambda value for this datapoint:
      #$line = <FID>;
      #$line = <FID>;
      #if ($line =~ m/\s+\S\s*(.+) \s*\S+ \s*\S+\)/)
      #{
      #  PDL::Core::set $dummyMatrix1,1,$n1,$1;
      #  #print "$1\n";
      #}

      $n1++;
    }
  }   
  CORE::close(FID);
  $n1--;  
  $dummyMatrix2 = pdl $dummyMatrix1->slice("0:1,0:$n1");
  #print $dummyMatrix2;
  # Sort the matrix in ascending number datapoints
  $ix = qsorti (pdl transpose ($dummyMatrix2->slice("0,0:$n1")));
  my $dataMatrix = zeroes($dummyMatrix2->getdim(0),$n1+1);
  for ($n = 0; $n <= $n1; $n++)
  {
    $dummy = at($ix,$n,0);
    $tmp = $dataMatrix->slice("0:1,$n");
    $tmp .= $dummyMatrix2->slice("0:1,$dummy");
  }
  #print $dataMatrix;
  my $nErrorPoints = 0;
  my $errorPointMatrix = zeroes(2,$badDataPoints->getdim(1));
  my $meanError = avg($dataMatrix->slice("1,:"));
  my $treshold = 5*$meanError;
  #print "\nThreshold\n", $treshold, "\n\n\n";
  for ($n = 0; $n <= $n1; $n++)
  {
    if (at($dataMatrix,1,$n) > $treshold)
    {
      # The $n and datapoint may not correspond due to unprojected datapoints.
      # Indexing is therefore a little tricky:
      PDL::Core::set $badDataPoints,0,(at($dataMatrix,0,$n)-1),0;
      $tmp = $errorPointMatrix->slice(":,$nErrorPoints"); 
      $tmp .= $dataMatrix->slice(":,$n");
      $nErrorPoints++;
      #print RESULTFID $dataMatrix->slice(":,$n");
      #print (at($dataMatrix,0,$n)-1), "\n"; 
    }
  }
  $nErrorPoints--;
  print RESULTFID "\nDatapoint with too high error:\n";
  if ($nErrorPoints < 0)
  {
    print RESULTFID "No points exceeded treshold for bad projections.\n";
  }
  else
  {
    print RESULTFID $errorPointMatrix->slice(":,0:$nErrorPoints");
  }
  #print $meanError, "\n";

  my $surfaceMatrix = zeroes(2,$badDataPoints->getdim(1)); # datapoint number, and lambda value
  $filename = "surfacePositions.ipdata";
  my $datapoint;
  my $lambda;
  $n = 0;
  CORE::open(FID, $filename) || die "Couldn't open $filename file";
  $line = <FID>;
  while ($line = <FID>) 
  {
    ($dataPoint, $lambda) = ($line =~ m/(\d+)\s+(\S+)/);
    PDL::Core::set $surfaceMatrix,0,$n,$dataPoint;
    PDL::Core::set $surfaceMatrix,1,$n,$lambda;
    if ($lambda == 0) # No projection was obtained for dataPoint
    {
      PDL::Core::set $badDataPoints,0,$n,0;
    }
    #print "$dataPoint  $lambda\n";
    $n++;
  }   
  CORE::close(FID);
  if ($n != $badDataPoints->getdim(1))
  {
    print "Warning: Number of datapoints may not be correct in $filename. 
           This may effect the calculation of error in projection.\n";
  }
  $n--;  
  print RESULTFID "\n readResult resultat: SurfaceMatrix:", $surfaceMatrix;

  return($surfaceMatrix->slice("1,0:$n"), $badDataPoints);
}



sub removeBadPoints
{
  # Gets matrix, removes the rows with bad points and return  
  # the edited matrix

  my ($matrixIn, $badDataPoints) = @_;
  my $nDataPoints = $badDataPoints->getdim(1);
  my $vector = pdl $badDataPoints;
  my $n;
  my $k = 0;

  reshape $vector,$nDataPoints;
  my $indx = which($vector == 0);
  my $matrixOut = zeroes($matrixIn->getdim(0),$matrixIn->getdim(1)-nelem($indx));
  for ($n=0; $n < $nDataPoints; $n++)
  {
    if (at($badDataPoints,0,$n) == 1)
    {
      $tmp = $matrixOut->slice(":,$k"); 
      $tmp .= $matrixIn->slice(":,$n");
      $k++;
    }
  }
  return $matrixOut;
}

# Subroutine readMeasurements
# The routine reads in the lambda values from the file lambdaValues.txt which is
# the conversion from the rc data in endo2.ipdata into prolate coordinates.
# Returns a vector with the lambda values.
sub readMeasurements
{
  my $filename = "lambdaValues.txt";
  my $line;
  my $n = 0;
  my $y = zeroes(1,3000); #initial guess of number of datapoints

  CORE::open(FID, $filename) || die "Couldn't open $filename file";
  while ($line = <FID>) 
  {
    if ($line =~ m/\ (\d+)\s*(\S+)/)
    {
      #print "$1 $2\n";
      PDL::Core::set $y,0,$n,$2;
      $n++;
    }
  }
  CORE::close(FID);
  $n--;
  return($y->slice("0,0:$n"));
}


sub printBadPoints
{
  #my (RESULTFID) = @_;
  my ($badDataPoints, $vector) = @_;
  my $n = 0;

  print RESULTFID "\nListing of bad points that are not included in error calculation:\n";
  for ($n = 0; $n < $badDataPoints->getdim(1); $n++)
  {
    if (at($badDataPoints,0,$n) == 0)
    {
      print RESULTFID sprintf(" %3d  %f\n", $n+1, at($vector,0,$n));
    }
  } 
  print RESULTFID "Listing of bad points finnished.\n\n";

  return(1);
}


