#!/usr/bin/perl -w
#
# This script creates meshes from the scp data sets.
#
#  

use File::Copy;
use Math::Trig;
#$CMISS_EXAMPLES = '/dept/eng_sci/stevens/examples/';
$CMISS_EXAMPLES = '/var/tmp/carey/examples';
$DATA_DIR = "${CMISS_EXAMPLES}/8/81/811/8117/cmiss_input/";
$MESH_DIR = "${CMISS_EXAMPLES}/8/81/811/8118/ouput/";
$SCRIPT_DIR = "${CMISS_EXAMPLES}/8/81/811/8118/cmiss_input/";
#$CMISS = '/cmiss1/cmiss/cm/bin/cmo_n32';
#$CMGUI = '/cmiss1/cmiss/bin/cmgui';

$CMISS = '/usr/local/bin/cmo';
$CMGUI = '/product/cmiss/cmgui/bin/cmgui_optimised';

#@SPECIMENS = qw/scp02 scp04 scp05 scp09 scp10/;
@SPECIMENS = qw/scp04/;
$ELEMENTS_AROUND = 8;
$ELEMENTS_HIGH = 4;

chdir $MESH_DIR;

# Create a directory for each specimen if necessary
foreach my $specimen (@SPECIMENS)
{
  mkdir $specimen,0777 unless -d $specimen;
}

@SURFACES = qw/epi endo/;
#@SURFACES = qw/epi /;

# Put the transformed data into each directory
foreach my $specimen (@SPECIMENS)
{
  foreach my $surface (@SURFACES)
  {
    system "nawk -f ${SCRIPT_DIR}convert_data_xyz_yzx.awk ${DATA_DIR}${specimen}/${specimen}${surface}.ipdata > ${specimen}/${specimen}${surface}.ipdata_tmp";
    system "nawk -f ${SCRIPT_DIR}renumber_data.awk ${specimen}/${specimen}${surface}.ipdata_tmp > ${specimen}/${specimen}${surface}.ipdata";
    system "rm ${specimen}/${specimen}${surface}.ipdata_tmp";
  }
}



# Put the files necessary for the fit in each directory
foreach my $specimen (@SPECIMENS)
{
  foreach my $file (qw(README fit.com data.ippara 3drc.ipcoor bicubic_fit.ipbase view.com refine1.com refine2.com refine3.com 3CHerm.ipbase update.com view_full.com))
  {
    copy("${SCRIPT_DIR}ipfiles/$file", $specimen);
  }

  chdir $specimen;

  foreach my $surface (@SURFACES)
  {
    if($surface eq "epi"){
      $INIT_RADIUS = 25;
    }else{
      $INIT_RADIUS = 15;
    }
    &make_initial_ipnode($specimen, $surface, $ELEMENTS_AROUND, $ELEMENTS_HIGH, $INIT_RADIUS);
    &make_initial_ipelem($specimen, $surface, $ELEMENTS_AROUND, $ELEMENTS_HIGH);
    &make_initial_ipfit($specimen, $surface, $ELEMENTS_AROUND, $ELEMENTS_HIGH);
    if($ELEMENTS_HIGH == 2){
      system "${CMISS} -exec \'\$SPECIMEN=\"${specimen}\" \; \$SURFACE=\"${surface}\"\' refine1.com";
    }
    if($ELEMENTS_HIGH == 4){
      system "${CMISS} -exec \'\$SPECIMEN=\"${specimen}\" \; \$SURFACE=\"${surface}\"\' refine2.com";
    }
    if($ELEMENTS_HIGH == 5){
      system "${CMISS} -exec \'\$SPECIMEN=\"${specimen}\" \; \$SURFACE=\"${surface}\"\' refine3.com";
    }
  }
  chdir "..";
}



# Fit the surfaces
foreach my $specimen (@SPECIMENS)
{
  chdir $specimen;
  foreach my $surface (@SURFACES)
  {
    system "${CMISS} -exec \'\$SPECIMEN=\"${specimen}\" \; \$SURFACE=\"${surface}\"\' fit.com";
    system "${CMGUI} -exec \'\$SPECIMEN=\"${specimen}\" \; \$SURFACE=\"${surface}\"\' view.com";
  }
  chdir "..";
}



# Join surfaces
foreach my $specimen (@SPECIMENS)
{
  chdir $specimen;
  &connect_surfaces($specimen, $ELEMENTS_AROUND, $ELEMENTS_HIGH);
  system "${CMISS} -exec \'\$SPECIMEN=\"${specimen}\"' update.com";
  system "${CMGUI} -exec \'\$SPECIMEN=\"${specimen}\"' view_full.com";
  chdir "..";
}



# Subroutines ====================================================================================

sub make_initial_ipnode
{
  my $speci = shift;
  my $surf = shift;

  my $around = shift;
  my $high = shift;
  my $radius = shift;
  my $nodes = $around+1;  
  my $degree_interval = 360/$around;
  my $radians_interval = deg2rad($degree_interval);

  my $input = "$speci$surf.ipdata";
  my $output = "$speci$surf.ipnode";

  open(INPUT, "<$input") || die "Could not open file $output: $!\n";
  open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";

  $line = <INPUT>;
  $line = <INPUT>;
  my @fields = split (' ',$line);
  my $z = $fields[3];
  $base = 1;
  $x_tilt = 0;
  $y_tilt = 0;
  $num_data_points = 0;
  while ($base)
  {
    $line = <INPUT>;
    @fields = split (' ',$line);
    if($fields[3] == $z)
    {
      $x_tilt = $x_tilt+$fields[1];
      $y_tilt = $y_tilt+$fields[2];
      $num_data_points++;
    }
    else
    {
      $base = 0;
    }
  }
  $x_tilt = $x_tilt/$num_data_points;
  $y_tilt = $y_tilt/$num_data_points;
  
  print OUTPUT <<EOF;
 CMISS Version 1.21 ipnode File Version 2
 Heading: 

 The number of nodes is [    0]:    $nodes
 Number of coordinates [3]: 3
 Do you want prompting for different versions of nj=1 [N]? y
 Do you want prompting for different versions of nj=2 [N]? y
 Do you want prompting for different versions of nj=3 [N]? y
 The number of derivatives for coordinate 1 is [0]: 3
 The number of derivatives for coordinate 2 is [0]: 3
 The number of derivatives for coordinate 3 is [0]: 3

EOF

  for(my $node = 1; $node < $nodes; $node++)
  {
    my $x = (cos($radians_interval*$node)*$radius)+$x_tilt;
    my $y = (sin($radians_interval*$node)*$radius)+$y_tilt;

    print OUTPUT <<EOF;
 Node number [    0]:    $node
 The number of versions for nj=1 is [1]: 1
 The Xj(1) coordinate is [ 0.00000E+00]:   $x
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0
 The number of versions for nj=2 is [1]: 1
 The Xj(2) coordinate is [ 0.00000E+00]:   $y
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0
 The number of versions for nj=3 is [1]: 1
 The Xj(3) coordinate is [ 0.00000E+00]:   $z
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0

EOF
  }

  while (defined($line = <INPUT>))
  {
    @fields = split (' ',$line);
    $apex = $fields[3];
  }

    print OUTPUT <<EOF;
 Node number [    0]:    ${nodes}
 The number of versions for nj=1 is [1]: ${around}
EOF
  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number ${version}:
 The Xj(1) coordinate is [ 0.00000E+00]:   0
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0
EOF
  }

    print OUTPUT <<EOF;
 The number of versions for nj=2 is [1]: ${around}
EOF
  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number ${version}:
 The Xj(2) coordinate is [ 0.00000E+00]:   0
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0
EOF
  }

    print OUTPUT <<EOF;
 The number of versions for nj=3 is [1]: ${around}
EOF
  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number ${version}:
 The Xj(3) coordinate is [ 0.00000E+00]:  ${apex} 
 The derivative wrt direction 1 is [ 0.00000E+00]:   0
 The derivative wrt direction 2 is [ 0.00000E+00]:   0
 The derivative wrt directions 1 & 2 is [ 0.00000E+00]:   0
EOF
  }

  
  close (INPUT);
  close (OUTPUT);
}




sub make_initial_ipelem
{
  my $speci = shift;
  my $surf = shift;

  my $around = shift;
  my $high = shift;
  my $nodes = $around+1;  

  my $output = "$speci$surf.ipelem";

  open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";

  print OUTPUT <<EOF;
 CMISS Version 1.21 ipelem File Version 2
 Heading: Bi-cubic-Hermite
 
 The number of elements is [1]:    $around
 
EOF

  my $nn3 = 1;
  my $nn4 = 2;
  for(my $element = 1; $element <= $around; $element++)
  {

    print OUTPUT <<EOF;
 Element number [    1]:     $element
 The number of geometric Xj-coordinates is [3]: 3
 The basis function type for geometric variable 1 is [1]:  1
 The basis function type for geometric variable 2 is [1]:  1
 The basis function type for geometric variable 3 is [1]:  1
 Enter the 4 global numbers for basis 1:  $nodes  $nodes  $nn4 $nn3  
 The version number for occurrence  1 of node     $nodes, njj=1 is [ 1]: $nn4
 The version number for occurrence  1 of node     $nodes, njj=2 is [ 1]: $nn4
 The version number for occurrence  1 of node     $nodes, njj=3 is [ 1]: $nn4
 The version number for occurrence  2 of node     $nodes, njj=1 is [ 1]: $nn3
 The version number for occurrence  2 of node     $nodes, njj=2 is [ 1]: $nn3
 The version number for occurrence  2 of node     $nodes, njj=3 is [ 1]: $nn3

EOF
    $nn3++;
    $nn4++;
    if($nn4 == $nodes)
    {
      $nn4 = 1;
    }
  }

  close (OUTPUT);
}




sub make_initial_ipfit
{
  my $speci = shift;
  my $surf = shift;

  my $around = shift;
  my $high = shift;
  my $nodes = $around+1;  

  my $output = "$speci$surf.ipfit";

  open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";

  print OUTPUT <<EOF;
 CMISS Version 1.21 ipfit File Version 2
 Heading: 
 
 Specify whether problem is (1) linear or (2) nonlinear [1]: 1
 Specify the number of fitting problems [3]: 1
 Specify #geometric vars to be fitted for problem 1 [1]: 3
 Specify field# to store var 1 of problem 1 [1]: 1
 Specify field# to store var 2 of problem 1 [2]: 2
 Specify field# to store var 3 of problem 1 [3]: 3
 Specify geometric# to be fitted for var 1 of problem 1 [1]: 1
 Specify geometric# to be fitted for var 2 of problem 1 [2]: 2
 Specify geometric# to be fitted for var 3 of problem 1 [3]: 3
 Enter smoothing type [0]:
   (0) None
   (1) Sobolev on field
   (2) Sobolev on deviation from initial field
   (3) Strain energy
    1
 Enter element #s/name [EXIT]: all
 The 5 weights on derivs wrt Xi_1/_11/_2/_22/_12 are [prev]:
    5E-1 5E-1 5E-1 5E-1 10E-3
 Enter element #s/name [EXIT]: 0
 
 Do you want the global matrices stored as sparse matrices [Y]? n
 
 Do you want to enter the coupling coefficients [N]? N
 
 Enter node #s/name to fix [EXIT]: 1..33
 Are any variables for variable 1 of fit variable 1 fixed [N]?: n
 Are any variables for variable 2 of fit variable 1 fixed [N]?: n
 Are any variables for variable 3 of fit variable 1 fixed [N]?: y
 Is variable 3 of fit variable 1 fixed [N]?: y
 Is variable 3 of fit variable 1 derivative 2 fixed [N]?: n
 Is variable 3 of fit variable 1 derivative 3 fixed [N]?: n
 Is variable 3 of fit variable 1 derivative 4 fixed [N]?: n

 Enter node #s/name to fix [EXIT]: $nodes
 Are any variables for variable 1 of fit variable 1 fixed [N]?: y
EOF

  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number  ${version}:
 Is variable 1 of fit variable 1 fixed [N]?: y
 Is variable 1 of fit variable 1 derivative 2 fixed [N]?: N
 Is variable 1 of fit variable 1 derivative 3 fixed [N]?: N
 Is variable 1 of fit variable 1 derivative 4 fixed [N]?: N
EOF
  }

    print OUTPUT <<EOF;
 Are any variables for variable 2 of fit variable 1 fixed [N]?: y
EOF
  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number  ${version}:
 Is variable 2 of fit variable 1 fixed [N]?: y
 Is variable 2 of fit variable 1 derivative 2 fixed [N]?: N
 Is variable 2 of fit variable 1 derivative 3 fixed [N]?: N
 Is variable 2 of fit variable 1 derivative 4 fixed [N]?: N
EOF
  }

    print OUTPUT <<EOF;
 Are any variables for variable 3 of fit variable 1 fixed [N]?: y
EOF
  for(my $version = 1; $version <= $around; $version++)
  {
    print OUTPUT <<EOF;
 For version number  ${version}:
 Is variable 3 of fit variable 1 fixed [N]?: y
 Is variable 3 of fit variable 1 derivative 2 fixed [N]?: N
 Is variable 3 of fit variable 1 derivative 3 fixed [N]?: N
 Is variable 3 of fit variable 1 derivative 4 fixed [N]?: N
EOF
  }

  print OUTPUT <<EOF;
 
 Enter node #s/name to fix [EXIT]: 0
 Specify type of linear solution procedure [1]:
   (1) LU Factorisation
  *(2) SVD Factorisation
   (3) Biconjugate gradient
   (4) Generalised minimum residual
   (5) Least Squares
   (6) Conjugate gradient
   (7) Unused
    1
 Do you want the solution matrix stored as a sparse matrix [Y]? n
 Specify option for linear solution [0]:
   (0) No output
   (1) Timing only
   (2) & Solver output
   (3) & Global solution matrices
   (4) & Global stiffness matrices
   (5) & Element matrices
    0
EOF


  close (OUTPUT);
}










sub connect_surfaces
{
  my $speci = shift;
  my $around = shift;
  my $high = shift;
  my $apex = $around+1;  

  my $epi_node_input = "${speci}epi_fitted.ipnode";
  my $endo_node_input = "${speci}endo_fitted.ipnode";
  my $endo_elem_input = "${speci}endo_fitted.ipelem";
  my $node_output = "$speci.ipnode";
  my $elem_output = "$speci.ipelem";

  # Nodes
  my $number_of_nodes_line = ' The number of nodes is';
  my $nodes_per_surface;
  my $nodes;

  open(INPUT1, "<$endo_node_input") || die "Could not open file : $endo_node_input!\n";
  open(INPUT2, "<$epi_node_input") || die "Could not open file : $epi_node_input!\n";
  open(OUTPUT, ">$node_output") || die "Could not open file : $node_output!\n";
  
  while (defined($line = <INPUT1>))
  {
    my @fields = split (' ',$line);
    my $last = $#fields;
    if ($line =~ m|^\Q$number_of_nodes_line\E|)
    {
      $nodes_per_surface = $fields[$last];
      $fields[$last] = $nodes_per_surface*2;
      print OUTPUT " @fields\n";
    }else{
      print OUTPUT $line;
    }
  }

  $nodes = $nodes_per_surface;
  my $node_number_line = ' Node number';
  while (defined($line = <INPUT2>))
  {
    my @fields = split (' ',$line);
    my $last = $#fields;
    if ($line =~ m|^\Q$node_number_line\E|)
    {
      $nodes++;
      $fields[$last] = $nodes;
      print OUTPUT " @fields\n";
    }else{
      print OUTPUT $line;
    }
  }

  close (INPUT1);
  close (INPUT2);
  close (OUTPUT);



  # Elements
  my $line_to_change = " Enter the 4 ";
  my $version_line = " The version number for occurrence";
  open(INPUT, "<$endo_elem_input") || die "Could not open file : $endo_elem_input!\n";
  open(OUTPUT, ">$elem_output") || die "Could not open file : $elem_output!\n";

  $count = 0;
  $version = 0;
  while (defined($line = <INPUT>))
  {
    my @fields = split (' ',$line);
    my $last = $#fields;

     if ($line =~ m|^\Q$line_to_change\E|)
    {
      $n1 = $fields[$last-3];
      $n2 = $fields[$last-2];
      $n3 = $fields[$last-1];
      $n4 = $fields[$last];
      $n5 = $fields[$last-3]+$nodes_per_surface;
      $n6 = $fields[$last-2]+$nodes_per_surface;
      $n7 = $fields[$last-1]+$nodes_per_surface;
      $n8 = $fields[$last]+$nodes_per_surface;
      print OUTPUT " Enter the 8 global numbers for basis 1: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
      if($fields[$last-3] != $fields[$last-2]){
        print OUTPUT " Enter the 8 numbers for basis 2 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
        print OUTPUT " Enter the 8 numbers for basis 3 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
        print OUTPUT " Enter the 8 numbers for basis 8 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
      }

    }
    elsif ($line =~ m|^\Q$version_line\E|)
    {
      print OUTPUT $line;
      $count++;
    }
    else
    {
      print OUTPUT $line;
    }
    if($count == 6){
      $version++;
      if($version != $around){
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=1 is [ 1]:  ",$version+1,"\n";
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=2 is [ 1]:  ",$version+1,"\n";
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=3 is [ 1]:  ",$version+1,"\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=1 is [ 1]:  ",$version,"\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=2 is [ 1]:  ",$version,"\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=3 is [ 1]:  ",$version,"\n";
      }else{
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=1 is [ 1]:  1\n";
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=2 is [ 1]:  1\n";
        print OUTPUT " The version number for occurrence  1 of node    ",$apex+$nodes_per_surface,", njj=3 is [ 1]:  1\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=1 is [ 1]:  ",$version,"\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=2 is [ 1]:  ",$version,"\n";
        print OUTPUT " The version number for occurrence  2 of node    ",$apex+$nodes_per_surface,", njj=3 is [ 1]:  ",$version,"\n";
      }
      print OUTPUT " Enter the 8 numbers for basis 2 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
      print OUTPUT " Enter the 8 numbers for basis 3 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
      print OUTPUT " Enter the 8 numbers for basis 8 [prev]: ",$n1," ",$n2," ",$n3," ",$n4," ",$n5," ",$n6," ",$n7," ",$n8,"\n";
      $count = 0;
    }
  } 

  close (INPUT);
  close (OUTPUT);


}


