$SPECIMEN="full_heart"


$TRUE = 1;  # Initialise logicals
$FALSE = 0;


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                       Execution Flow                         \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
set num_threads 1;
#
# Output 
#
$OUTPUT_DIR = "output/";
if( ! -d ${OUTPUT_DIR})
{
		mkdir ${OUTPUT_DIR};
}
#
# Solve for or read in deformations
#
$SOLVE_INFLATION = $FALSE;
#
$SOLVE_ISOVOLUMIC_CONTRACTION = $TRUE;
#
$SOLVE_EJECTION = $FALSE;
#
$SOLVE_ISOVOLUMIC_RELAXATION = $FALSE;
#
if ( $SOLVE_ISOVOLUMIC_CONTRACTION || $SOLVE_EJECTION || $SOLVE_ISOVOLUMIC_RELAXATION )
{
		$COUPLED_PROBLEM = $TRUE;
} else {
		$COUPLED_PROBLEM = $FALSE;
}


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                    Regions & Classes                         \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
# Regions
#
$WALL = 1;
$LV_CAVITY = 2;
$RV_CAVITY = 3;#
#
# Classes
#
$MECHANICS = 1;
$CELLULAR = 2;
$STRESS_FIT = 3;
$MATERIAL_FIT = 4;


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                       Environment                            \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#fem def para;r;final
fem def para;r;scp04
fem def coor;r;final


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m     Nodes, Elements & Fibre Fields for Ventricular Wall      \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
fem def node;r;test;
#
fem def base;r;full;                                     # 1..8 Geometry and fibres
fem define;add bases;r;3Linear-2PressAuxXi3_4x4x4Gauss;  # 9 Pressure
#
fem def elem;r;test;
#
fem define base;r;fullreadse;                            #redefine bases to read in scale factors 
fem define;add bases;r;3Linear-2PressAuxXi3_4x4x4Gauss 
fem def line;r;test
#
#fem define fibre;r;test
#fem define elem;r;test fibre
#
# Node Groups
#
fem group nodes 9..12,21..22,35..36,42,44,46,48,50,52,54,56,82,84,96,98,100,102 as FIXED;
#
# Element Groups
#
fem group elements 6,11,14..17,28,35,37 as LV_FREE_WALL; 
fem group elements 14..21,32..34,37,43,51..52 as SEPTUM;
fem group elements 22..27,36,45..50,55,56,81 as RV_FREE_WALL;



    #fem update nodes hanging nodes 144,146 element 47 direction 1;
    fem export nodes;junk as heart;
    fem export elements;junk as wall;
    quit

set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                     Equations                                \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
#fem update nodes hanging nodes 144,146 element 47 direction 1;
fem define equation;r;finelas_tch_incomp lockregion $WALL;
fem define mapping;r;test3;


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                     Material Properties                      \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
fem define material;r;holmes region $WALL;
fem define active;r;active0_00 region $WALL;


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                      Initial Conditions                      \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
fem define initial;r;test region $WALL;


set echo;
#==========================================================================================================
print "\033[0;30;42m ============================================================ \033[0m\n"; 
print "\033[0;30;42m                       Solution Method                        \033[0m\n"; 
print "\033[0;30;42m ============================================================ \033[0m\n"; 
#==========================================================================================================
set echo;
#
fem define solve;r;newton region $WALL;


set echo; 
if ( $SOLVE_INFLATION ) {
		#==========================================================================================================
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		print "\033[0;30;42m                       Solve Inflation                        \033[0m\n"; 
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		#==========================================================================================================
		print "\033[0;30;43m Increase cavity pressures incrementally to simulate diastole \033[0m\n"; 
    #
		set out;steps_inflate on;
		$NAME1 = ${SPECIMEN}."_press_";
		$TOL = 0.001;
    $MAXIMUM_INCREM = 4; 
    $PRESS = 0.0;
		$ITERS = 8;

		for $i ( 0..$MAXIMUM_INCREM ) 
		{
				if ($i < 1) 
				{
            $INCREM = 0.0;
				} else {
            $INCREM = 0.1;
        }
        $PRESS = $PRESS + $INCREM;
        $PRESS_mmHg = ${PRESS}*7.500637554192106;
				$FILENAME = ${NAME1}.${PRESS}; 
        fem solve increment $INCREM iter $ITERS error $TOL ;
        fem define initial;w;${OUTPUT_DIR}.${FILENAME};
        fem export nodes;${OUTPUT_DIR}.${FILENAME}  as heart;
        fem export nodes;${OUTPUT_DIR}${FILENAME}_def field as heart;
        fem export elements;${OUTPUT_DIR}.${FILENAME} as heart;
        fem export elements;${OUTPUT_DIR}${FILENAME}_def field as wall;
    }
} else {
		#==========================================================================================================
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		print "\033[0;30;42m                    End-Diastolic State                       \033[0m\n"; 
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		#==========================================================================================================
    #
    $FILENAME = "full_heart_press_0.4";
    fem define initial;r;${OUTPUT_DIR}.${FILENAME} region $WALL;
    fem define solve;r;newton region $WALL;
    #
    fem solve increment 0 iter 8 error 0.001;
    #
    fem export nodes;${OUTPUT_DIR}.${FILENAME} as heart;
    fem export nodes;${OUTPUT_DIR}${FILENAME}_def field as heart;
    fem export elements;${OUTPUT_DIR}.${FILENAME} as wall;
    fem export elements;${OUTPUT_DIR}${FILENAME}_def field as wall;
}
set echo; 


set echo;
if ( $COUPLED_PROBLEM ) {           # Set up coupled regions
		#==========================================================================================================
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		print "\033[0;30;42m                    Set up Coupled Regions                    \033[0m\n"; 
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		#==========================================================================================================
		#
		# Save scale factors for the ventricular wall elements
		#
		fem def base;r;full;                                    # redefine bases to calculate scale factors 
		fem define;add bases;r;3Linear-2PressAuxXi3_4x4x4Gauss;  
		fem define;add bases;r;Auxillary;                       # Auxillary basis for pressure coupling
		#
		# Define geometry and dependent variable information for LV and RV cavities
		#
    fem define region;r;coupled;
    #
		fem define coord;r;lv_cavity region $LV_CAVITY;
		fem define node;r;lv_cavity region $LV_CAVITY;
		fem define element;r;lv_cavity region $LV_CAVITY;
    #
#		fem define coord;r;rv_cavity region $RV_CAVITY;
#		fem define node;r;rv_cavity region $RV_CAVITY;
#		fem define element;r;rv_cavity region $RV_CAVITY;
    #
		fem export nodes;final as all_nodes region all;
		fem export elements;wall as wall region $WALL;
		fem export elements;lv_cavity as lv_cavity region $LV_CAVITY;
#		fem export elements;rv_cavity as rv_cavity region $RV_CAVITY;
    #
		fem group elem 119..127 as lv_cavity_elems region $LV_CAVITY;
#		fem group elem 137..145 as lv_cavity_base_elems region $LV_CAVITY;
		fem group elem  119..127 as lv_cavity_base_elems region $LV_CAVITY;
		fem group nodes xi2=1 xi3=1 elem lv_cavity_base_elems as lv_cavity_base_nodes region $LV_CAVITY;
#		fem group elem 151..160 as rv_cavity_elems region $RV_CAVITY;
		fem define equation;r;coupled region $WALL,$LV_CAVITY lock;
		fem define material;r;cavity region $LV_CAVITY;
#		fem define material;r;cavity region $RV_CAVITY;
		fem define initial;r;lv_cavity region $LV_CAVITY;
#		fem define initial;r;rv_cavity region $RV_CAVITY;
#		#
#		# Define cavity/wall coupling and solution information
#		#
		fem define coupling;r;coupled;
		fem define solve;r;coupled coupled region $WALL,$LV_CAVITY;
		fem update solution coupled source_region $WALL;
		fem update solution cavity_reference average 152 in 3 node lv_cavity_base_nodes region $LV_CAVITY;
#		fem update solution cavity_reference average 203 in $NHlambda node 14,18 region $RV_CAVITY;
		#
    # Read back in scale factors for the ventricular wall elements
		#
		fem define base;r;fullreadse;                            # redefine bases to read in scale factors 
		fem define;add bases;r;3Linear-2PressAuxXi3_4x4x4Gauss; 
		fem define;add bases;r;Auxillary;                       
    #
    fem def line;r;test region $WALL; 
		fem define lines;r;lv_cavity region $LV_CAVITY;          # zero length scalefactors in xi 2 at apex
    #
		fem export nodes;${OUTPUT_DIR}heart as heart region all;          
    fem export elements;${OUTPUT_DIR}lv_cavity as lv_cavity region $LV_CAVITY;
#    fem export elements;${OUTPUT_DIR}rv_cavity as rv_cavity region $RV_CAVITY;
    fem export elements;${OUTPUT_DIR}wall as wall region $WALL;
    fem export nodes;${OUTPUT_DIR}heart_def field as heart region all;          
    fem export elements;${OUTPUT_DIR}lv_cavity_def field as lv_cavity region $LV_CAVITY;
#    fem export elements;${OUTPUT_DIR}rv_cavity_def field as rv_cavity region $RV_CAVITY;
    fem export elements;${OUTPUT_DIR}wall_def field as wall region $WALL;
}
set echo;


set echo;
if ( $SOLVE_ISOVOLUMIC_CONTRACTION ) {
		#==========================================================================================================
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		print "\033[0;30;42m             Solving Isovolumic Contraction                   \033[0m\n"; 
		print "\033[0;30;42m ============================================================ \033[0m\n"; 
		#==========================================================================================================
		set output;${OUTPUT_DIR}steps_contract on;
    $NAME1 = "active0_";
    $NAME2 = "fullheart_def__active_";
    $INCREM = 0.0;
    $ITER = 20;
    $ERROR_TOLERANCE = 0.001;
    $MAX_PRESS=10;
    #
    # Isovolumically contract the ventricles by incrementally
    # increasing intracellular calcium concentration and solving.
    if ( $MAX_PRESS == 10 ) 
    {
				$MAX_CA = 24;
		} elsif ( $MAX_PRESS == 30 ) {
				$MAX_CA = 30;
		} else {
				fem quit; # required activation level unknown
		}
    for $CA ( 1..$MAX_CA )
    {
				$NUMZEROS=2-length($CA);
				$CANUM="";
				for $Z ( 1..$NUMZEROS ) 
				{
						$CANUM=$CANUM."0";
				}
				$CANUM=$CANUM.$CA;
				print "\033[0;30;43m                    Calcium Level = ${CA}                         \033[0m\n"; 
				$FILENAME=${NAME2}.${CANUM};
        fem define active;r;${NAME1}.${CANUM} region $WALL;
        fem solve increment $INCREM iter $ITER coupled error $ERROR_TOLERANCE;
        fem define initial;w;${OUTPUT_DIR}.${FILENAME} region $WALL,$LV_CAVITY,$RV_CAVITY;
        fem export nodes;${OUTPUT_DIR}.${FILENAME} as heart region all;
        fem export nodes;${OUTPUT_DIR}${FILENAME}_def field as heart region all;
    }
    set out off;
}
set echo;


#fem quit
