# Adds element_number_offset and sets new_xi3 for all element:xi points in
# .exdata file
sub make_shifted_seed_points
{
	my $element_number_offset = shift;
	my $new_xi3 = shift;
	my $line;
	my $input = "seed_points.exdata";
	my $output = "new_seed_points.exdata";
	my $new_element_number;

	CORE::open(INPUT, "<$input") || die "Could not open file $input: $!\n";
	CORE::open(OUTPUT, ">$output") || die "Could not open file $output: $!\n";
	while (defined($line = <INPUT>))
	{
		if ($line =~ m/(\s*E\s+)(\d+)(\s+3\s+\S+\s+\S+\s+)(\S+)(\s*)/)
		{
			$new_element_number = $2 + $element_number_offset;
			print OUTPUT "$1$new_element_number$3$new_xi3\n";
		} else
		{
			print OUTPUT "$line";
		}
	}
	close (INPUT);
	close (OUTPUT);
}


#Global settings
open comfile global.com exec;

$ventricles = "ventricles";
$endocardium = "endocardium";

# read in the geometry from inside out
gfx read node reference_heart0000.exnode;
gfx read elem reference_heart.exelem;

# create materials
gfx create material epicardium ambient 0.2 0 0 diffuse 0.61 0.01 0.02 emission 0 0 0 specular 0.05 0.05 0.05 alpha 0.25 shininess 0.8
gfx create material endocardium ambient 0.28 0 0.33 diffuse 0.33 0.22 0.56 emission 0 0 0 specular 0.5 0.5 0.5 alpha 0.5 shininess 0.8
gfx cre material fibres ambient 0.5 0 0 diffuse 0.9 0 0 specular 0.5 0.5 0.5 shininess 0.8

# create groups of elements in inner and outer heart layers
gfx create egroup inner_heart add 31..60,91..120
gfx create egroup outer_heart add 1..30,61..90

gfx modify g_element heart general clear circle_discretization 6 default_coordinate default_coordinate element_discretization "4*4*4" native_discretization none
gfx modify g_element heart surfaces exterior face xi2_1 select_on material epicardium selected_material default_selected render_shaded

gfx modify g_element inner_heart general clear circle_discretization 6 default_coordinate default_coordinate element_discretization "12*12*12" native_discretization none
gfx modify g_element inner_heart surfaces exterior face xi3_1 select_on material endocardium selected_material default_selected render_shaded
gfx modify g_element inner_heart surfaces exterior face xi3_0 select_on material endocardium selected_material default_selected render_shaded

gfx modify g_element outer_heart general clear circle_discretization 6 default_coordinate default_coordinate element_discretization "12*12*12" native_discretization none
gfx modify g_element outer_heart surfaces exterior face xi3_0 select_on material endocardium selected_material default_selected render_shaded
gfx modify g_element outer_heart surfaces exterior face xi3_1 select_on material epicardium selected_material default_selected render_shaded

# create a data group for storing streamline seed points
gfx create dgroup seed_points

# set up window
gfx create window 1 double_buffer
gfx modify window 1 layout simple ortho_axes -x -y eye_spacing 0.25 width $width height $height
gfx modify window 1 background colour 0 0 0
gfx modify window 1 set slow_transparency
open com view exec
if ($antialias)
  {
	 gfx modify window 1 set antialias $antialias;
  } else
  {
	 gfx modify window 1 set no_antialias;
  }


#gfx define field density composite 0.04;
# previous code for getting random points on an isosurface xi3=0.9
#gfx define field xi3 composite xi.3
#$iso_xi_value = 0.9;
#gfx create iso_surface as fibre_surface iso_scalar xi3 iso_value $iso_xi_value surface_data_density density surface_data_group seed_points from outer_heart with 4;

gfx define field fibre_axes fibre_axes fibre reference_fibres coordinate reference_coordinates

$number_of_iterations = 50;
$angle = 360.0 / $number_of_iterations;
$i = 0;
$moviename = "heart_fibres.movie";
print "Creating movie $moviename\n";
#gfx movie create $moviename rle24_sgi_movie3 win 1
$frame = 0;

for ($i = 0 ; $i < $number_of_iterations ; $i++)
{
	$xi3 = 1.0 - ($i + 0.5) / $number_of_iterations;
	make_shifted_seed_points(0,$xi3);
	gfx read data new_seed_points;
	gfx create streamlines as muscle_fibres seed_data_field element_xi seed_data_group seed_points vector fibre_axes material fibres rectangle length 16 width 0.8 from heart;
	gfx draw muscle_fibres;
	gfx print win 1 rgb file $moviename.$frame.rgb;
	$frame++;
#	gfx movie add_frame;
	gfx mod win 1 image rotate -1 0 0 $angle;
}

for ($i = 0 ; $i < $number_of_iterations ; $i++)
{
	$xi3 = 1.0 - ($i + 0.5) / $number_of_iterations;
	make_shifted_seed_points(30,$xi3);
	gfx read data new_seed_points;
	gfx create streamlines as muscle_fibres seed_data_field element_xi seed_data_group seed_points vector fibre_axes material fibres rectangle length 16 width 0.8 from heart;
	gfx draw muscle_fibres;
	gfx print win 1 rgb file $moviename.$frame.rgb;
	$frame++;
#	gfx movie add_frame;
	gfx mod win 1 image rotate -1 0 0 $angle;
}

#gfx movie end;


open com view exec

$moviename = "heart_fibres_static.movie";
print "Creating movie $moviename\n";
#gfx movie create $moviename rle24_sgi_movie3 win 1
$frame = 0;

for ($i = 0 ; $i < $number_of_iterations ; $i++)
{
	$xi3 = 1.0 - ($i + 0.5) / $number_of_iterations;
	make_shifted_seed_points(0,$xi3);
	gfx read data new_seed_points;
	gfx create streamlines as muscle_fibres seed_data_field element_xi seed_data_group seed_points vector fibre_axes material fibres rectangle length 16 width 0.8 from heart;
	gfx draw muscle_fibres;
	gfx print win 1 rgb file $moviename.$frame.rgb;
	$frame++;
#	gfx movie add_frame;
}

for ($i = 0 ; $i < $number_of_iterations ; $i++)
{
	$xi3 = 1.0 - ($i + 0.5) / $number_of_iterations;
	make_shifted_seed_points(30,$xi3);
	gfx read data new_seed_points;
	gfx create streamlines as muscle_fibres seed_data_field element_xi seed_data_group seed_points vector fibre_axes material fibres rectangle length 16 width 0.8 from heart;
	gfx draw muscle_fibres;
	gfx print win 1 rgb file $moviename.$frame.rgb;
	$frame++;
#	gfx movie add_frame;
}

#gfx movie end;
quit
