  #
  # EJECTION
  #
  # Increase LV pressure until it is above or equal to measured systolic Plv
  # Increase calcium concentration gradually to max = 1.
  # change mu until it reaches set limit
  fem define initial;w;$output."temp" region $WALL
  $Plv = MyFunc::readPressure($output."wall_endIVC.ipinit");
  print "LV Pressure: ", $Plv, "\n";
  $Cactn = MyFunc::findCalcium($output."calciumEndIVC.ipacti");
  print "Calcium = ", $Cactn, "\n";

  $return = MyFunc::editInitFile(1);
  fem define initial;r;$output."ejection" reg $WALL
  fem define solve;r;solve reg $WALL
  fem eval resid wrt geom reg $WALL
  fem solve increment 0.0 iter 20 error 1.0e-1

  $EF_at_peak_P=50; #NB not the same as total EF that also include reduced ejection phase
  $V_at_peak_P = $DIASTOLIC_LV_CAVITY_VOLUME*(100-$EF_at_peak_P)/100;
  print "Diastolic LV volume:     $DIASTOLIC_LV_CAVITY_VOLUME\n";
  print "Volume at peak pressure: $V_at_peak_P\n"; 
  $LV_CAVITY_VOLUME = $DIASTOLIC_LV_CAVITY_VOLUME;
  #fem list elements deformed total cavity_volume region $LV_CAVITY;
  print "\nLV cavity volume = $LV_CAVITY_VOLUME\n";
  $LV_END_IVC_VOLUME = $LV_CAVITY_VOLUME;
  $LV_PRESS = $Plv;
  $P_start_ejection = $LV_PRESS;
  $V_of_P = MyFunc::findVolumeGivenP($LV_PRESS,$LV_END_IVC_VOLUME,$EF_at_peak_P,$P_start_ejection,$Psystole);
  print "Current LV P = $LV_PRESS\n"; 
  print "Volume for this pressure should be $V_of_P\n"; 

  $pIncr = 0.1; #kPa
  print "Pressure increment: $pIncr\n";
  $INCREM = $pIncr;


    #==========================================================================================================
    print "\033[0;30;42m ============================================================ \033[0m\n"; 
    print "\033[0;30;42m                     Solving Ejection                         \033[0m\n"; 
    print "\033[0;30;42m ============================================================ \033[0m\n"; 
    #==========================================================================================================
    #
    # Decrement the cavity pressures to simulate ejection
    #
    #
    $ITER = 30;
    $ERROR_TOLERANCE = 0.1;

    while ($LV_PRESS < $Psystole )
    {
      $LV_PRESS = $LV_PRESS + $INCREM;
      $V_of_P = MyFunc::findVolumeGivenP($LV_PRESS,$LV_END_IVC_VOLUME,$EF_at_peak_P,$P_start_ejection,$Psystole);
      print "Solving for new LV_PRESS = $LV_PRESS\n"; 
      print "Volume for this pressure should be $V_of_P\n"; 
      fem solve increment $INCREM iter $ITER  error $ERROR_TOLERANCE;
      fem list elements deformed total cavity_volume region $LV_CAVITY;
      print "\nLV cavity volume = ${LV_CAVITY_VOLUME}\n";
      print "Volume for this pressure should be $V_of_P\n"; 

          $name = "heartEjection_".${LV_PRESS};
          $file = $output.$name
          fem export nodes;$file field as $name offset 0
          fem export elements;$file field as $name offset_elem 0
          $name = "heartEjectionStress_".${LV_PRESS}; 
          $file = $output.$name
          fem update gauss stress total Cauchy reg $WALL
          #fem update gauss stress total_no_hydro Piola reg $WALL
          fem export gauss;$file yg as gauss_stress offset 0 reg $WALL
          $name = "heartEjectionStrain_".${LV_PRESS};
          $file = $output.$name
          fem update gauss strain fibre extension_ratios reg $WALL
          fem export gauss;$file yg as gauss_strain reg $WALL

      $errorFac=0.01;
      print "V_of_P: $V_of_P\n";
      print "Must lie between: ", $V_of_P - $V_of_P*$errorFac, " and ", $V_of_P + $V_of_P*$errorFac, "\n";
      print "LV_CAVITY_VOLUME:    $LV_CAVITY_VOLUME\n\n";
      
      $bFirst = $TRUE;
      $bPlussIncr = $FALSE;
      $bMinusIncr = $FALSE;
      $small_ca_incr = 0.001;
      $loops = 0;
      while ((abs($LV_CAVITY_VOLUME - $V_of_P) > $V_of_P*$errorFac) && ($loops<10))
      {
        if($bFirst)
        {
          print "Increasing calcium to reduce volume\n";
          $Cactn=$Cactn+0.0025;
          $bFirst = $FALSE;
        } elsif($LV_CAVITY_VOLUME - $V_of_P > 0)
        {
          print "Increasing calcium to reduce volume\n";
          if($bPlussIncr && $bMinusIncr)
          {
            $small_ca_incr = $small_ca_incr/2;
          }
          $bPlussIncr = $TRUE;
          $Cactn=$Cactn+$small_ca_incr;
        } elsif($LV_CAVITY_VOLUME - $V_of_P < 0)
        {
          print "Decreasing calcium to increase volume\n";
          if($bPlussIncr && $bMinusIncr)
          {
            $small_ca_incr = $small_ca_incr/2;
          }
          $bMinusIncr = $TRUE;
          $Cactn=$Cactn-$small_ca_incr;
        } else
        {
          print "Error: Something is wrong!\n";
        }
        $loops++;
        
        MyFunc::changeCalcium($Cactn);
        fem define active;r;"calcium0xx"
        print "\033[0;30;43m                New Calcium Level = ${Cactn}                         \033[0m\n"; 
        fem solve increment 0 iter $ITER error $ERROR_TOLERANCE;
        #
          $name = "heartEjection_".${LV_PRESS};
          $file = $output.$name
          fem export nodes;$file field as $name offset 0
          fem export elements;$file field as $name offset_elem 0
          $name = "heartEjectionStress_".${LV_PRESS}; 
          $file = $output.$name
          fem update gauss stress total Cauchy reg $WALL
          #fem update gauss stress total_no_hydro Piola reg $WALL
          fem export gauss;$file yg as gauss_stress offset 0 reg $WALL
          $name = "heartEjectionStrain_".${LV_PRESS};
          $file = $output.$name
          fem update gauss strain fibre extension_ratios reg $WALL
          fem export gauss;$file yg as gauss_strain reg $WALL
        #
        fem list elements deformed total cavity_volume region $LV_CAVITY;
        print "\nLV cavity volume = ${LV_CAVITY_VOLUME}\n";
        print "Must lie between: ", $V_of_P - $V_of_P*$errorFac, " and ", $V_of_P + $V_of_P*$errorFac, "\n";
        print "Volume for this pressure should be $V_of_P\n";
        print "LV pressure = $LV_PRESS\n";
        print "Solved for Calcium level = $Cactn\n\n";
      }
      #fem solve increment 0 iter 7 error $ERROR_TOLERANCE;
      fem define initial;w;$outputInit."lv_def_ejection_$time" reg $WALL
      $return = MyFunc::writeCalciumLevel($Cactn,$time);
      $time=$time+1;
      fem list elements deformed total cavity_volume region $LV_CAVITY;
      print "\033[0;30;43m                Results for this load step                        \033[0m\n"; 
      print "\nLV cavity volume for this load step = ${LV_CAVITY_VOLUME}\n";
      print "Volume for this pressure should be $V_of_P\n";
      print "Must lie between: ", $V_of_P - $V_of_P*$errorFac, " and ", $V_of_P + $V_of_P*$errorFac, "\n";
      print "LV pressure = $LV_PRESS\n";
      print "Solved for Calcium level = $Cactn\n\n";      
    }

      
  $name = "heartEjection_".${LV_PRESS};
  $file = $output.$name
  fem export nodes;$file field as $name offset 0
  fem export elements;$file field as $name offset_elem 0
  $name = "heartEjectionStress_".${LV_PRESS}; 
  $file = $output.$name
  fem update gauss stress total Cauchy reg $WALL
  #fem update gauss stress total_no_hydro Piola reg $WALL
  fem export gauss;$file yg as gauss_stress offset 0 reg $WALL
  $name = "heartEjectionStrain_".${LV_PRESS};
  $file = $output.$name
  fem update gauss strain fibre extension_ratios reg $WALL
  fem export gauss;$file yg as gauss_strain reg $WALL
 
  fem define initial;w;$output."endRapidEjection" reg $WALL
  fem define active;w;$output."calciumEndRapidEjection"

  read com;export
  print "Last time step: $time\n";

