  fem define initial;w;$output."temp" region $WALL
  $Plv = MyFunc::readPressure($output."endEjection.ipinit");
  print "LV Pressure: ", $Plv, "\n";
  $Cactn = MyFunc::findCalcium($output."calciumEndEjection.ipacti");
  print "Calcium = ", $Cactn, "\n";


  # Set up cavity region geometry
  fem define region;r;regions
  fem define coord;r;cavityCoord region $LV_CAVITY
  fem define node;r;cavity region $LV_CAVITY
  fem define element;r;cavity2 region $LV_CAVITY

  # Set up equations, material properties and initial conditions
  # for cavity region
  fem define equation;r;equationRegions region $WALL,$LV_CAVITY lock
  fem define material;r;materialCavityIVC"region $LV_CAVITY
  fem define initial;r;initCavity region $LV_CAVITY
  #
  # Define cavity-to-wall coupling and solution procedure information
  fem define coupling;r;coupled
  fem define solve;r;solveRegion coupled region $WALL,$LV_CAVITY
  #
  # Transfer wall deformation from the inflation problem to 
  # geometric dependent variables for the cavity region
  fem update solution coupled source_region $WALL
  fem update solution cavity_reference average 104 in 1 node 14..17 region $LV_CAVITY

  fem evaluate residuals wrt geom_params coupled
  #fem evaluate reactions coupled

  fem list element total region $LV_CAVITY
  fem list element deformed total region $LV_CAVITY

  # ############################################
  # Start decreasing calcium
  # ############################################
  $iterations = 30
  $boolContinue = 1;
  fem solve coupled increment 0 iter $iterations error 1.0e-3
  while ( $boolContinue )
  {
    if(($Plv-$Pla)<1.0)
    {
      $Cactn = $Cactn - 0.005;
    } else
    {
      $Cactn = $Cactn - 0.02;
    }
    print "New Calcium level = ", $Cactn, "\n";
    $return = MyFunc::changeCalcium($Cactn);
    fem define active;r;"calcium0xx"
    fem solve coupled increment 0 iter $iterations error 1.0e-3
    fem define initial;w;$outputInit."lv_def_ivr_$time" reg $WALL
    $return = MyFunc::writeCalciumLevel($Cactn,$time);
    $time=$time+1;

    read com;export

    fem define initial;w;$output."temp" region $WALL
    $Plv = MyFunc::readPressure($output."temp.ipinit");
    print "LV Pressure: ", $Plv, "\n";
    if(($Plv < $Pla) || ($Cactn<=0.14))
    {
      $boolContinue=0;
    }
  }

  # Save deformed configurations for end isovolumic contraction
  fem define initial;w;$output."wall_endIVR" region $WALL
  fem define initial;w;$output."cavity_endIVR" region $LV_CAVITY
  fem define initial;w;$output."bothReg_endIVR" region $WALL,$LV_CAVITY
  fem define active;w;$output."calciumEndIVR"

  fem list element total region $LV_CAVITY
  fem list element deformed total region $LV_CAVITY

  print "Last time step: $time\n";
  
  
