# Used for steady-state current plots

set xzeroaxis lt -1
set yzeroaxis lt -1
set grid

ENa = 40
EK  = -110
EK1 = -30
Esi = 70
ECl = -70
Es  = -52

gNa = 150
gsi = 0.8
gsi_ = 0.04
gNab = 0.105
gClb = 0.01

am(V) = (V+47)/(1-exp(-(V+47.13)/10))
bm(V) = 40*exp(-(V+72)/17.86)
m_inf(V) = am(V)/(am(V)+bm(V))

ah(V) = 0.0085*exp(-(V+71)/5.435)
bh(V) = 2.5/(exp(-(V+10)/12.2)+1)
h_inf(V) = ah(V)/(ah(V)+bh(V))

INa_inf(V) = gNa * m_inf(V)**3 * h_inf(V) * (V-ENa)

ad(V) = (V+40)/(500*(1-exp(-(V+40)/10)))
bd(V) = 0.02*exp(-(V+40)/11.26)
d_inf(V) = ad(V)/(ad(V)+bd(V))

af(V) = 0.000987*exp(-(V+60)/25)
bf(V) = 0.02/(exp(-(V+26)/11.5)+1)
f_inf(V) = af(V)/(af(V)+bf(V))

d1(V) = 1/(1+exp(-(V+40)/6.667))

Isi_inf(V) = gsi*d_inf(V)*f_inf(V)*(V-Esi) + gsi_*d1(V)*(V-Esi)

as(V) = 0.001*(V-Es)/(1-exp(-(V-Es)/5))
bs(V) = 0.00005*exp(-(V-Es)/15)
s_inf(V) = as(V)/(as(V)+bs(V))

IK2(V) = 2.8*(exp((V-EK)/25)-1)/(exp((V+60)/12.5)+exp((V+60)/25))
IK2_inf(V) = IK2(V)*s_inf(V)


ax1(V) = 0.0005*exp((V+50)/12.1)/(1+exp((V+50)/17.5))
bx1(V) = 0.0013*exp(-(V+20)/16.67)/(1+exp(-(V+20)/20))
x1_inf(V) = ax1(V)/(ax1(V)+bx1(V))

Ix1(V) = 1.2*(exp((V+95)/25)-1)/exp((V+45)/25)
Ix1_inf(V) = Ix1(V)*x1_inf(V)

ax2(V) = 0.000127/(1+exp(-(V+19)/5))
bx2(V) = 0.0003*exp(-(V+20)/16.67)/(1+exp(-(V+20)/25))
x2_inf(V) = ax2(V)/(ax2(V)+bx2(V))

Ix2(V) = 25+0.385*V
Ix2_inf(V) = Ix2(V)*x2_inf(V)

IK1(V) = IK2(V)/2.8 + 0.2*(V-EK1)/(1-exp(-(V-EK1)/25))
INab(V) = gNab*(V-ENa)
IClb(V) = gClb*(V-ECl)

I1(V) = INab(V)
I2(V) = INab(V) + INa_inf(V)
I3(V) = INab(V) + INa_inf(V) + Isi_inf(V)
I4(V) = INab(V) + INa_inf(V) + Isi_inf(V) + IClb(V)

I5(V) = I4(V) + IK1(V)
I6(V) = I5(V) + IK2_inf(V)
I7(V) = I6(V) + Ix1_inf(V)
I8(V) = I7(V) + Ix2_inf(V)

I9(V) = I4(V) + IK1(V) + IK2(V) + Ix1_inf(V) + Ix2_inf(V)

#plot [-120:80] IK2(x) title "IK2" lt 1,\
#               IK1(x) title "IK1" lt 2,\
#               Ix1(x) title "Ix1" lt 3,\
#               Ix2(x) title "Ix2" lt 4

plot [-120:80] I4(x),I5(x),I6(x),I7(x),I8(x),I9(x)
