# ........................................................................

# Name : jcs.pl

# Author : Justin Fernandez (jw.fernandez@auckland.ac.nz)

# Date : 9.10.2003

# Last Modified : 22.10.2003

# Description : Reads in orthotrak static and dynamic files and calculates
#               joint angles and translations according to the joint
#               coordinate system (jcs).


# Usage : Perl math lib (Math:Bigfloat, Math::Trig, Math::Matrix)

# ........................................................................
BEGIN { push @INC, "/home/soe/jfer004/perl_routines/"; }

use Math::BigFloat;
use Math::Trig;
# Math matrix.pm is located in /home/soe/jfer004/perl_routines/Math/
use Math::Matrix;

# subroutines
sub normalise{
@vector = @_;
$norm = Math::BigFloat->new(($vector[0]*$vector[0])+($vector[1]*$vector[1])+($vector[2]*$vector[2]));
$norm=$norm->fsqrt(3);
@vector=(($vector[0]/$norm)->fround(6),($vector[1]/$norm)->fround(6),($vector[2]/$norm)->fround(6));
return @vector; 
}

sub cross{
@vector = @_;
$cross_vector[0]=($vector[5]*$vector[1])-($vector[4]*$vector[2]);
$cross_vector[1]=-($vector[5]*$vector[0])+($vector[3]*$vector[2]);
$cross_vector[2]=($vector[4]*$vector[0])-($vector[3]*$vector[1]);
return @cross_vector; 
}

sub dot{
@dot_vector = @_;
$dot_scalar=($dot_vector[0]*$dot_vector[3])+($dot_vector[1]*$dot_vector[4])+($dot_vector[2]*$dot_vector[5]);
return return $dot_scalar; 
}

sub sign{
@sign_vector = @_;
$sign_scalar=($dot_vector[0]*$dot_vector[3])+($dot_vector[1]*$dot_vector[4])+($dot_vector[2]*$dot_vector[5]);
if ($sign_scalar>0){$sign_scalar_result=1};
if ($sign_scalar<0){$sign_scalar_result=-1};
return return $sign_scalar_result; 
}

sub rot_markertolab{
@vector = @_;
$rot_vector[0]=($vector[9]*$vector[0])+($vector[10]*$vector[1])+($vector[11]*$vector[2]);
$rot_vector[1]=($vector[12]*$vector[0])+($vector[13]*$vector[1])+($vector[14]*$vector[2]);
$rot_vector[2]=($vector[15]*$vector[0])+($vector[16]*$vector[1])+($vector[17]*$vector[2]);
$rot_vector[3]=($vector[9]*$vector[3])+($vector[10]*$vector[4])+($vector[11]*$vector[5]);
$rot_vector[4]=($vector[12]*$vector[3])+($vector[13]*$vector[4])+($vector[14]*$vector[5]);
$rot_vector[5]=($vector[15]*$vector[3])+($vector[16]*$vector[4])+($vector[17]*$vector[5]);
$rot_vector[6]=($vector[9]*$vector[6])+($vector[10]*$vector[7])+($vector[11]*$vector[8]);
$rot_vector[7]=($vector[12]*$vector[6])+($vector[13]*$vector[7])+($vector[14]*$vector[8]);
$rot_vector[8]=($vector[15]*$vector[6])+($vector[16]*$vector[7])+($vector[17]*$vector[8]);
return @rot_vector; 
}

sub rot_labtomarker{
@vector = @_;
$rot_vector[0]=($vector[9]*$vector[0])+($vector[10]*$vector[1])+($vector[11]*$vector[2]);
$rot_vector[1]=($vector[9]*$vector[3])+($vector[10]*$vector[4])+($vector[11]*$vector[5]);
$rot_vector[2]=($vector[9]*$vector[6])+($vector[10]*$vector[7])+($vector[11]*$vector[8]);
$rot_vector[3]=($vector[12]*$vector[0])+($vector[13]*$vector[1])+($vector[14]*$vector[2]);
$rot_vector[4]=($vector[12]*$vector[3])+($vector[13]*$vector[4])+($vector[14]*$vector[5]);
$rot_vector[5]=($vector[12]*$vector[6])+($vector[13]*$vector[7])+($vector[14]*$vector[8]);
$rot_vector[6]=($vector[15]*$vector[0])+($vector[16]*$vector[1])+($vector[17]*$vector[2]);
$rot_vector[7]=($vector[15]*$vector[3])+($vector[16]*$vector[4])+($vector[17]*$vector[5]);
$rot_vector[8]=($vector[15]*$vector[6])+($vector[16]*$vector[7])+($vector[17]*$vector[8]);
return @rot_vector; 
}

sub generate_condyle_rel{
@vector1 = @_;
$con_vector1[0]=($vector1[0]*($vector1[9]))+($vector1[1]*($vector1[10]))+($vector1[2]*($vector1[11]));
$con_vector1[1]=($vector1[3]*($vector1[9]))+($vector1[4]*($vector1[10]))+($vector1[5]*($vector1[11]));
$con_vector1[2]=($vector1[6]*($vector1[9]))+($vector1[7]*($vector1[10]))+($vector1[8]*($vector1[11]));
$con_vector1[0]=$con_vector1[0]-($vector1[0]*($vector1[12]))-($vector1[1]*($vector1[13]))-($vector1[2]*($vector1[14]));
$con_vector1[1]=$con_vector1[1]-($vector1[3]*($vector1[12]))-($vector1[4]*($vector1[13]))-($vector1[5]*($vector1[14]));
$con_vector1[2]=$con_vector1[2]-($vector1[6]*($vector1[12]))-($vector1[7]*($vector1[13]))-($vector1[8]*($vector1[14]));
return @con_vector1; 
}

sub generate_condyle_lab{
@vector2 = @_;
$con_vector2[0]=($vector2[0]*$vector2[9])+($vector2[1]*$vector2[10])+($vector2[2]*$vector2[11])+$vector2[12];
$con_vector2[1]=($vector2[3]*$vector2[9])+($vector2[4]*$vector2[10])+($vector2[5]*$vector2[11])+$vector2[13];
$con_vector2[2]=($vector2[6]*$vector2[9])+($vector2[7]*$vector2[10])+($vector2[8]*$vector2[11])+$vector2[14];
return @con_vector2; 
}

sub flexion_extension{
@flex_ext_vec = @_;
$flex_ext_angle=($flex_ext_vec[0]*$flex_ext_vec[3])+($flex_ext_vec[1]*$flex_ext_vec[4])+($flex_ext_vec[2]*$flex_ext_vec[5]);
$flex_ext_angle=acos($flex_ext_angle);
return $flex_ext_angle; 
}

sub axial_rotation{
@axi_rot_vec = @_;
$axi_rot_angle=($axi_rot_vec[0]*$axi_rot_vec[3])+($axi_rot_vec[1]*$axi_rot_vec[4])+($axi_rot_vec[2]*$axi_rot_vec[5]);
$axi_rot_angle=acos($axi_rot_angle);
return $axi_rot_angle; 
}

sub abduction_adduction{
@abd_add_vec = @_;
$abd_add_angle=($abd_add_vec[0]*$abd_add_vec[3])+($abd_add_vec[1]*$abd_add_vec[4])+($abd_add_vec[2]*$abd_add_vec[5]);
$abd_add_angle=acos($abd_add_angle);
return $abd_add_angle; 
}

# get input arguments
$input_file1 = $ARGV[0];
$output_file1 = "angles_static.txt";
$output_file2 = "static.exdata";
$output_file3 = "static_vector.exdata";

# Open input file1
open (input_text1,$input_file1) or die "Error opening ";

# Open output file1.
open (output_text1, ">$output_file1") or die "Error opening ";

# Open output file2.
open (output_exdata1, ">$output_file2") or die "Error opening ";

# Open output file3.
open (output_exdata2, ">$output_file3") or die "Error opening ";

# read in static text file to array 'coordstatic'
@coordstatic=();
$i=0;
while ($line = <input_text1>) 
{           
  @components = split(/\s+/,$line); 
  for $j (0..$#components)
  {          
    $coordstatic[$i][$j]=$components[$j];
  }
  $i++;   
}

# Close the files.
close input_text1;

# assign body components to associated arrays
# So far only hip, femur and tibia implemented for left leg.

# dynamic markers - always present
@l_asis=($coordstatic[5][11],$coordstatic[5][12],$coordstatic[5][13]);
$landmarks[1][1]=$coordstatic[5][11];
$landmarks[1][2]=$coordstatic[5][12];
$landmarks[1][3]=$coordstatic[5][13];
@v_sacral=($coordstatic[5][38],$coordstatic[5][39],$coordstatic[5][40]);
$landmarks[2][1]=$coordstatic[5][38];
$landmarks[2][2]=$coordstatic[5][39];
$landmarks[2][3]=$coordstatic[5][40];
@r_asis=($coordstatic[5][50],$coordstatic[5][51],$coordstatic[5][52]);
$landmarks[3][1]=$coordstatic[5][50];
$landmarks[3][2]=$coordstatic[5][51];
$landmarks[3][3]=$coordstatic[5][52];
@thigh_marker1=($coordstatic[5][14],$coordstatic[5][15],$coordstatic[5][16]);
$landmarks[4][1]=$coordstatic[5][14];
$landmarks[4][2]=$coordstatic[5][15];
$landmarks[4][3]=$coordstatic[5][16];
@thigh_marker2=($coordstatic[5][17],$coordstatic[5][18],$coordstatic[5][19]);
$landmarks[5][1]=$coordstatic[5][17];
$landmarks[5][2]=$coordstatic[5][18];
$landmarks[5][3]=$coordstatic[5][19];
@thigh_marker3=($coordstatic[5][20],$coordstatic[5][21],$coordstatic[5][22]);
$landmarks[6][1]=$coordstatic[5][20];
$landmarks[6][2]=$coordstatic[5][21];
$landmarks[6][3]=$coordstatic[5][22];
@shank_marker1=($coordstatic[5][23],$coordstatic[5][24],$coordstatic[5][25]);
$landmarks[7][1]=$coordstatic[5][23];
$landmarks[7][2]=$coordstatic[5][24];
$landmarks[7][3]=$coordstatic[5][25];
@shank_marker2=($coordstatic[5][26],$coordstatic[5][27],$coordstatic[5][28]);
$landmarks[8][1]=$coordstatic[5][26];
$landmarks[8][2]=$coordstatic[5][27];
$landmarks[8][3]=$coordstatic[5][28];
@shank_marker3=($coordstatic[5][29],$coordstatic[5][30],$coordstatic[5][31]);
$landmarks[9][1]=$coordstatic[5][29];
$landmarks[9][2]=$coordstatic[5][30];
$landmarks[9][3]=$coordstatic[5][31];
# static markers - only in static trial
@lat_knee=($coordstatic[5][77],$coordstatic[5][78],$coordstatic[5][79]);
$landmarks[10][1]=$coordstatic[5][77];
$landmarks[10][2]=$coordstatic[5][78];
$landmarks[10][3]=$coordstatic[5][79];
@med_knee=($coordstatic[5][80],$coordstatic[5][81],$coordstatic[5][82]);
$landmarks[11][1]=$coordstatic[5][80];
$landmarks[11][2]=$coordstatic[5][81];
$landmarks[11][3]=$coordstatic[5][82];
@lat_ankle=($coordstatic[5][83],$coordstatic[5][84],$coordstatic[5][85]);
$landmarks[12][1]=$coordstatic[5][83];
$landmarks[12][2]=$coordstatic[5][84];
$landmarks[12][3]=$coordstatic[5][85];
@med_ankle=($coordstatic[5][86],$coordstatic[5][87],$coordstatic[5][88]);
$landmarks[13][1]=$coordstatic[5][86];
$landmarks[13][2]=$coordstatic[5][87];
$landmarks[13][3]=$coordstatic[5][88];
# generated positions
$hip_center[0]=($l_asis[0]+$r_asis[0])/2;
$hip_center[1]=($l_asis[1]+$r_asis[1])/2;
$hip_center[2]=($l_asis[2]+$r_asis[2])/2;
$landmarks[14][1]=$hip_center[0];
$landmarks[14][2]=$hip_center[1];
$landmarks[14][3]=$hip_center[2];
$knee_center[0]=($lat_knee[0]+$med_knee[0])/2;
$knee_center[1]=($lat_knee[1]+$med_knee[1])/2;
$knee_center[2]=($lat_knee[2]+$med_knee[2])/2;
$landmarks[15][1]=$knee_center[0];
$landmarks[15][2]=$knee_center[1];
$landmarks[15][3]=$knee_center[2];
$ankle_center[0]=($lat_ankle[0]+$med_ankle[0])/2;
$ankle_center[1]=($lat_ankle[1]+$med_ankle[1])/2;
$ankle_center[2]=($lat_ankle[2]+$med_ankle[2])/2;
$landmarks[16][1]=$ankle_center[0];
$landmarks[16][2]=$ankle_center[1];
$landmarks[16][3]=$ankle_center[2];

# work out pelvic coordinate system
$hip_y[0]=$l_asis[0]-$hip_center[0];
$hip_y[1]=$l_asis[1]-$hip_center[1];
$hip_y[2]=$l_asis[2]-$hip_center[2];

$hip_x[0]=$v_sacral[0]-$hip_center[0];
$hip_x[1]=$v_sacral[1]-$hip_center[1];
$hip_x[2]=$v_sacral[2]-$hip_center[2];

# normalise x and y
@hip_y_norm=&normalise(@hip_y);
@hip_x_norm=&normalise(@hip_x);

# cross product of x and y gives z vector
@hip_z=&cross(@hip_x_norm,@hip_y_norm);
@hip_z_norm=&normalise(@hip_z);

# cross product of y and z gives new x (overwrite previous x vector)
@hip_x=&cross(@hip_y_norm,@hip_z_norm);
@hip_x_norm=&normalise(@hip_x);

# Assis distance
$assis_d = Math::BigFloat->new(($l_asis[0]-$r_asis[0])**2+($l_asis[1]-$r_asis[1])**2+($l_asis[2]-$r_asis[2])**2);
$assis_d=$assis_d->fsqrt(3);

# Work out femur ball center
$xd=0.22*$assis_d;
$yd=0.32*$assis_d;
$zd=0.34*$assis_d;
$femur_ball[0]=$hip_center[0]+($xd*$hip_x_norm[0])+($yd*$hip_y_norm[0])+($zd*$hip_z_norm[0]);
$femur_ball[1]=$hip_center[1]+($xd*$hip_x_norm[1])+($yd*$hip_y_norm[1])+($zd*$hip_z_norm[1]);
$femur_ball[2]=$hip_center[2]+($xd*$hip_x_norm[2])+($yd*$hip_y_norm[2])+($zd*$hip_z_norm[2]);
$landmarks[17][1]=$femur_ball[0];
$landmarks[17][2]=$femur_ball[1];
$landmarks[17][3]=$femur_ball[2];

# work out thigh coordinate system
# uses femur ball center.
$thigh_x[0]=$femur_ball[0]-$knee_center[0];
$thigh_x[1]=$femur_ball[1]-$knee_center[1];
$thigh_x[2]=$femur_ball[2]-$knee_center[2];

$thigh_z[0]=$lat_knee[0]-$knee_center[0];
$thigh_z[1]=$lat_knee[1]-$knee_center[1];
$thigh_z[2]=$lat_knee[2]-$knee_center[2];

# normalise x and z
@thigh_x_norm=&normalise(@thigh_x);
@thigh_z_norm=&normalise(@thigh_z);

# cross product of z and x gives y vector
@thigh_y=&cross(@thigh_z_norm,@thigh_x_norm);
@thigh_y_norm=&normalise(@thigh_y);

# cross product of x and y gives new z (overwrite previous z vector)
@thigh_z=&cross(@thigh_x_norm,@thigh_y_norm);
@thigh_z_norm=&normalise(@thigh_z);

# work out shank coordinate system
# uses knee center.
$shank_x[0]=$knee_center[0]-$ankle_center[0];
$shank_x[1]=$knee_center[1]-$ankle_center[1];
$shank_x[2]=$knee_center[2]-$ankle_center[2];

$shank_z[0]=$lat_ankle[0]-$ankle_center[0];
$shank_z[1]=$lat_ankle[1]-$ankle_center[1];
$shank_z[2]=$lat_ankle[2]-$ankle_center[2];

# normalise x and z
@shank_x_norm=&normalise(@shank_x);
@shank_z_norm=&normalise(@shank_z);

# cross product of z and x gives y vector
@shank_y=&cross(@shank_z_norm,@shank_x_norm);
@shank_y_norm=&normalise(@shank_y);

# cross product of x and y gives new z (overwrite previous z vector)
@shank_z=&cross(@shank_x_norm,@shank_y_norm);
@shank_z_norm=&normalise(@shank_z);


# marker coordinate systems
# thigh
$th_mark_coord_center[0]=($thigh_marker2[0]+$thigh_marker3[0])/2;
$th_mark_coord_center[1]=($thigh_marker2[1]+$thigh_marker3[1])/2;
$th_mark_coord_center[2]=($thigh_marker2[2]+$thigh_marker3[2])/2;

# thigh marker coord x
$th_mark_coord_x[0]=$thigh_marker1[0]-$th_mark_coord_center[0];
$th_mark_coord_x[1]=$thigh_marker1[1]-$th_mark_coord_center[1];
$th_mark_coord_x[2]=$thigh_marker1[2]-$th_mark_coord_center[2];

# thigh marker coord z
$th_mark_coord_z[0]=$thigh_marker3[0]-$th_mark_coord_center[0];
$th_mark_coord_z[1]=$thigh_marker3[1]-$th_mark_coord_center[1];
$th_mark_coord_z[2]=$thigh_marker3[2]-$th_mark_coord_center[2];

# normalise x and z
@th_mark_coord_x_norm=&normalise(@th_mark_coord_x);
@th_mark_coord_z_norm=&normalise(@th_mark_coord_z);

# cross product of z and x gives y vector
@th_mark_coord_y=&cross(@th_mark_coord_z_norm,@th_mark_coord_x_norm);
@th_mark_coord_y_norm=&normalise(@th_mark_coord_y);

# cross product of x and y gives new z (overwrite previous z vector)
@th_mark_coord_z=&cross(@th_mark_coord_x_norm,@th_mark_coord_y_norm);
@th_mark_coord_z_norm=&normalise(@th_mark_coord_z);

# shank
$sh_mark_coord_center[0]=($shank_marker2[0]+$shank_marker3[0])/2;
$sh_mark_coord_center[1]=($shank_marker2[1]+$shank_marker3[1])/2;
$sh_mark_coord_center[2]=($shank_marker2[2]+$shank_marker3[2])/2;

# thigh marker coord x
$sh_mark_coord_x[0]=$shank_marker1[0]-$sh_mark_coord_center[0];
$sh_mark_coord_x[1]=$shank_marker1[1]-$sh_mark_coord_center[1];
$sh_mark_coord_x[2]=$shank_marker1[2]-$sh_mark_coord_center[2];

# thigh marker coord z
$sh_mark_coord_z[0]=$shank_marker3[0]-$sh_mark_coord_center[0];
$sh_mark_coord_z[1]=$shank_marker3[1]-$sh_mark_coord_center[1];
$sh_mark_coord_z[2]=$shank_marker3[2]-$sh_mark_coord_center[2];

# normalise x and z
@sh_mark_coord_x_norm=&normalise(@sh_mark_coord_x);
@sh_mark_coord_z_norm=&normalise(@sh_mark_coord_z);

# cross product of z and x gives y vector
@sh_mark_coord_y=&cross(@sh_mark_coord_z_norm,@sh_mark_coord_x_norm);
@sh_mark_coord_y_norm=&normalise(@sh_mark_coord_y);

# cross product of x and y gives new z (overwrite previous z vector)
@sh_mark_coord_z=&cross(@sh_mark_coord_x_norm,@sh_mark_coord_y_norm);
@sh_mark_coord_z_norm=&normalise(@sh_mark_coord_z);


# Work out rotation matrix to generate condyles from marker coordinate system.
@lab_x_norm=(1,0,0);
@lab_y_norm=(0,1,0);
@lab_z_norm=(0,0,1);
@rot_matrix=&rot_labtomarker(@lab_x_norm,@lab_y_norm,@lab_z_norm,@th_mark_coord_x_norm,@th_mark_coord_y_norm,@th_mark_coord_z_norm);

$origin_rel[0]=$th_mark_coord_center[0];
$origin_rel[1]=$th_mark_coord_center[1];
$origin_rel[2]=$th_mark_coord_center[2];

# knee condyles vector with respect to thigh marker system
@lat_knee_rel=&generate_condyle_rel(@rot_matrix,@lat_knee,@origin_rel);
@med_knee_rel=&generate_condyle_rel(@rot_matrix,@med_knee,@origin_rel);

$origin_rel[0]=$sh_mark_coord_center[0];
$origin_rel[1]=$sh_mark_coord_center[1];
$origin_rel[2]=$sh_mark_coord_center[2];

# ankle condyles vector with respect to shank marker system
@rot_matrix=&rot_labtomarker(@lab_x_norm,@lab_y_norm,@lab_z_norm,@sh_mark_coord_x_norm,@sh_mark_coord_y_norm,@sh_mark_coord_z_norm);
@lat_ankle_rel=&generate_condyle_rel(@rot_matrix,@lat_ankle,@origin_rel);
@med_ankle_rel=&generate_condyle_rel(@rot_matrix,@med_ankle,@origin_rel);

# temp : work out knee_center with respect to shank coordinate systems.
@shank_knee_center_rel=&generate_condyle_rel(@rot_matrix,@knee_center,@origin_rel); 

# regenerate condyles for knee
@rot_matrix=&rot_markertolab(@lab_x_norm,@lab_y_norm,@lab_z_norm,@th_mark_coord_x_norm,@th_mark_coord_y_norm,@th_mark_coord_z_norm);

$origin_rel[0]=$th_mark_coord_center[0];
$origin_rel[1]=$th_mark_coord_center[1];
$origin_rel[2]=$th_mark_coord_center[2];

@lat_knee_new=&generate_condyle_lab(@rot_matrix,@lat_knee_rel,@origin_rel);
@med_knee_new=&generate_condyle_lab(@rot_matrix,@med_knee_rel,@origin_rel);

# regenerate condyles for ankle
@rot_matrix=&rot_markertolab(@lab_x_norm,@lab_y_norm,@lab_z_norm,@sh_mark_coord_x_norm,@sh_mark_coord_y_norm,@sh_mark_coord_z_norm);

$origin_rel[0]=$sh_mark_coord_center[0];
$origin_rel[1]=$sh_mark_coord_center[1];
$origin_rel[2]=$sh_mark_coord_center[2];

@lat_ankle_new=&generate_condyle_lab(@rot_matrix,@lat_ankle_rel,@origin_rel);
@med_ankle_new=&generate_condyle_lab(@rot_matrix,@med_ankle_rel,@origin_rel);

# regenerate shank_knee_center
@shank_knee_center_new=&generate_condyle_lab(@rot_matrix,@shank_knee_center_rel,@origin_rel);

# test : re-generate condyles.
#print  "\n";
#print $lat_knee[0] . "\n";
#print $lat_knee[1] . "\n";
#print $lat_knee[2] . "\n";
#print $lat_knee_new[0] . "\n";
#print $lat_knee_new[1] . "\n";
#print $lat_knee_new[2] . "\n";
#print "\n";
#print $med_knee[0] . "\n";
#print $med_knee[1] . "\n";
#print $med_knee[2] . "\n";
#print $med_knee_new[0] . "\n";
#print $med_knee_new[1] . "\n";
#print $med_knee_new[2] . "\n";
#print "\n";
#print $lat_ankle[0] . "\n";
#print $lat_ankle[1] . "\n";
#print $lat_ankle[2] . "\n";
#print $lat_ankle_new[0] . "\n";
#print $lat_ankle_new[1] . "\n";
#print $lat_ankle_new[2] . "\n";
#print "\n";
#print $med_ankle[0] . "\n";
#print $med_ankle[1] . "\n";
#print $med_ankle[2] . "\n";
#print $med_ankle_new[0] . "\n";
#print $med_ankle_new[1] . "\n";
#print $med_ankle_new[2] . "\n";
#print "\n";
#print $knee_center[0] . "\n";
#print $knee_center[1] . "\n";
#print $knee_center[2] . "\n";
#print $shank_knee_center_new[0] . "\n";
#print $shank_knee_center_new[1] . "\n";
#print $shank_knee_center_new[2] . "\n";


# Define joint coordinate system (JCS) unit vectors
# Fixed body axis to femur z axis (e1)
@e1=@thigh_z_norm;
# Fixed body axis to tibia x axis (e3)
@e3=@shank_x_norm;
# Relative axis to e1 (e1_r) directed anteriously.
@e1_r=@thigh_y_norm;
# Relative axis to e2 (e2_r) directed anteriously.
@e3_r=@shank_y_norm;

# Floating axis perpendicular to e1 and e3 (e2)
@e2=&cross(@e3,@e1);
@e2=&normalise(@e2);

# Work out clinical translations.
# Take vector that joins origins of femur and shank local coordinate systems (Tr).
# This vector is with respect to laboratory reference frame.
# Project this onto the jcs vectors (e1,e2,e3) q1=Tr.e1, q2=Tr.e2, q3=Tr.e3 to give clinical translations.

$Tr[0]=$ankle_center[0]-$knee_center[0];
$Tr[1]=$ankle_center[1]-$knee_center[1];
$Tr[2]=$ankle_center[2]-$knee_center[2];

$q[0]=&dot(@Tr,@e1);
$q[1]=&dot(@Tr,@e2);
$q[2]=&dot(@Tr,@e3);

# Print out knee flexion angles
print output_text1 "flexion_extension" . "  " . "axial_rotation" . "  " . "abduction_adduction" . "  " . "q1" . "  " . "q2" . "  " . "q3". "\n";
$flex_ext_angle=&flexion_extension(@e2,@e1_r);
print output_text1 180-($flex_ext_angle*180/pi) . "  ";
$axi_rot_angle=&axial_rotation(@e2,@e3_r);
print output_text1 180-($axi_rot_angle*180/pi) . "  ";
$abd_add_angle=&abduction_adduction(@e1,@e3);
print output_text1 90-($abd_add_angle*180/pi) . "  ";
print output_text1 $q[0] . "  " . $q[1] . "  " . $q[2] . "\n";

#create static exdata file
print output_exdata1 " Group name: static" . "\n";
print output_exdata1 " #Fields=1" . "\n";
print output_exdata1 " 1) coordinates, coordinate, rectangular cartesian, #Components=3" . "\n";
print output_exdata1 "  x.  Value index=1, #Derivatives=0, #Versions=1" . "\n";
print output_exdata1 "  y.  Value index=2, #Derivatives=0, #Versions=1" . "\n";
print output_exdata1 "  z.  Value index=3, #Derivatives=0, #Versions=1" . "\n";
for $i (1..17)
{
print output_exdata1 " Node: " . $i . "\n";
print output_exdata1 " " . $landmarks[$i][1] . "\n";
print output_exdata1 " " . $landmarks[$i][2] . "\n";
print output_exdata1 " " . $landmarks[$i][3] . "\n";
}

#create static exdata file with coordinate systems vectors
print output_exdata2 " Group name: static_vector" . "\n";
print output_exdata2 " #Fields=2" . "\n";
print output_exdata2 " 1) coordinates, coordinate, rectangular cartesian, #Components=3" . "\n";
print output_exdata2 "  x.  Value index=1, #Derivatives=0" . "\n";
print output_exdata2 "  y.  Value index=2, #Derivatives=0" . "\n";
print output_exdata2 "  z.  Value index=3, #Derivatives=0" . "\n";
print output_exdata2 " 2) error, field, rectangular cartesian, #Components=3" . "\n";
print output_exdata2 "  x.  Value index= 4, #Derivatives=0" . "\n";
print output_exdata2 "  y.  Value index= 5, #Derivatives=0" . "\n";
print output_exdata2 "  z.  Value index= 6, #Derivatives=0" . "\n";
print output_exdata2  " Node:     101". "\n";
print output_exdata2   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
print output_exdata2   $thigh_x_norm[0] . " " . $thigh_x_norm[1] . " " . $thigh_x_norm[2]. "\n";
print output_exdata2  " Node:     102". "\n";
print output_exdata2   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
print output_exdata2   $thigh_y_norm[0] . " " . $thigh_y_norm[1] . " " . $thigh_y_norm[2]. "\n";
print output_exdata2  " Node:     103". "\n";
print output_exdata2   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
print output_exdata2   $thigh_z_norm[0] . " " . $thigh_z_norm[1] . " " . $thigh_z_norm[2]. "\n";
print output_exdata2  " Node:     104". "\n";
print output_exdata2   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
print output_exdata2   $hip_x_norm[0] . " " . $hip_x_norm[1] . " " . $hip_x_norm[2]. "\n";
print output_exdata2  " Node:     105". "\n";
print output_exdata2   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
print output_exdata2   $hip_y_norm[0] . " " . $hip_y_norm[1] . " " . $hip_y_norm[2]. "\n";
print output_exdata2  " Node:     106". "\n";
print output_exdata2   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
print output_exdata2   $hip_z_norm[0] . " " . $hip_z_norm[1] . " " . $hip_z_norm[2]. "\n";
print output_exdata2  " Node:     107". "\n";
print output_exdata2   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
print output_exdata2   $shank_x_norm[0] . " " . $shank_x_norm[1] . " " . $shank_x_norm[2]. "\n";
print output_exdata2  " Node:     108". "\n";
print output_exdata2   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
print output_exdata2   $shank_y_norm[0] . " " . $shank_y_norm[1] . " " . $shank_y_norm[2]. "\n";
print output_exdata2  " Node:     109". "\n";
print output_exdata2   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
print output_exdata2   $shank_z_norm[0] . " " . $shank_z_norm[1] . " " . $shank_z_norm[2]. "\n";
print output_exdata2  " Node:     110". "\n";
print output_exdata2   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
print output_exdata2   $th_mark_coord_x_norm[0] . " " . $th_mark_coord_x_norm[1] . " " . $th_mark_coord_x_norm[2] . "\n";
print output_exdata2  " Node:     111". "\n";
print output_exdata2   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
print output_exdata2   $th_mark_coord_y_norm[0] . " " . $th_mark_coord_y_norm[1] . " " . $th_mark_coord_y_norm[2]. "\n";
print output_exdata2  " Node:     112". "\n";
print output_exdata2   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
print output_exdata2   $th_mark_coord_z_norm[0] . " " . $th_mark_coord_z_norm[1] . " " . $th_mark_coord_z_norm[2] . "\n";
print output_exdata2  " Node:     113". "\n";
print output_exdata2   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
print output_exdata2   $sh_mark_coord_x_norm[0] . " " . $sh_mark_coord_x_norm[1] . " " . $sh_mark_coord_x_norm[2] . "\n";
print output_exdata2  " Node:     114". "\n";
print output_exdata2   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
print output_exdata2   $sh_mark_coord_y_norm[0] . " " . $sh_mark_coord_y_norm[1] . " " . $sh_mark_coord_y_norm[2]. "\n";
print output_exdata2  " Node:     115". "\n";
print output_exdata2   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
print output_exdata2   $sh_mark_coord_z_norm[0] . " " . $sh_mark_coord_z_norm[1] . " " . $sh_mark_coord_z_norm[2] . "\n";

# Close the files.
close output_text1;
close output_exdata1;
close output_exdata2;

#########################################################################################################################
# read in dynamic file.

@coorddynamic=();
$i=0;
$input_file2 = $ARGV[1];

# Open input file2.
open (input_text2,$input_file2) or die "Error opening ";

while ($line = <input_text2>) 
{           
  @components = split(/\s+/,$line); 
  for $j (0..$#components)
  {            
    $coorddynamic[$i][$j]=$components[$j];
  }
  $i++;   
}

# Close the files.
close input_text2;

$output_file4 = "angles_dynamic.txt";
# Open output file4.
open (output_text2, ">$output_file4") or die "Error opening ";

#gnuplot data files
$gnu_flex = "flex.dat";
$gnu_rot = "rot.dat";
$gnu_abd = "abd.dat";
# Open output 
open (output_gnu_flex, ">$gnu_flex") or die "Error opening ";
open (output_gnu_rot, ">$gnu_rot") or die "Error opening ";
open (output_gnu_abd, ">$gnu_abd") or die "Error opening ";
# Open output for cmiss
open (output_cmiss_angles, ">cmiss_angles.txt") or die "Error opening ";
open (output_cmiss_rot_tra, ">cmiss_rot_tra.txt") or die "Error opening ";

$k=0;
$q=270;
for ($z=89;$z<194;$z=$z+1) # flexion9
#for ($z=178;$z<388;$z=$z+1) # flexion9/2
#for ($z=890;$z<1940;$z=$z+1) # flexion9/10
#for ($z=84;$z<224;$z=$z+1) # flexion8
#for ($z=111;$z<271;$z=$z+1) # flexion7
#for ($z=105;$z<235;$z=$z+1) # flexion6
#for ($z=90;$z<226;$z=$z+1) # flexion5
#for ($z=60;$z<190;$z=$z+1) # flexion4
#for ($z=60;$z<160;$z=$z+1) # flexion3
#for ($z=76;$z<195;$z=$z+2) # flexion1
{
$k=$k+1;
$output_file5 = "dynamic" . $z . ".exdata";
$output_file6 = "dynamic_vector" . $z . ".exdata";

# Open output file5.
open (output_exdata3, ">$output_file5") or die "Error opening ";

# Open output file6.
open (output_exdata4, ">$output_file6") or die "Error opening ";
 

# first dataset is on row 5
$row=$z+5;
# temp : Assume that femur and hip do not move (ie take first position only)
$const_pos=$row;

# dynamic markers - always present
@l_asis=($coorddynamic[$const_pos][11],$coorddynamic[$const_pos][12],$coorddynamic[$const_pos][13]);
$landmarks[1][1]=$coorddynamic[$const_pos][11];
$landmarks[1][2]=$coorddynamic[$const_pos][12];
$landmarks[1][3]=$coorddynamic[$const_pos][13];
@v_sacral=($coorddynamic[$const_pos][38],$coorddynamic[$const_pos][39],$coorddynamic[$const_pos][40]);
$landmarks[2][1]=$coorddynamic[$const_pos][38];
$landmarks[2][2]=$coorddynamic[$const_pos][39];
$landmarks[2][3]=$coorddynamic[$const_pos][40];
@r_asis=($coorddynamic[$const_pos][50],$coorddynamic[$const_pos][51],$coorddynamic[$const_pos][52]);
$landmarks[3][1]=$coorddynamic[$const_pos][50];
$landmarks[3][2]=$coorddynamic[$const_pos][51];
$landmarks[3][3]=$coorddynamic[$const_pos][52];
@thigh_marker1=($coorddynamic[$const_pos][14],$coorddynamic[$const_pos][15],$coorddynamic[$const_pos][16]);
$landmarks[4][1]=$coorddynamic[$const_pos][14];
$landmarks[4][2]=$coorddynamic[$const_pos][15];
$landmarks[4][3]=$coorddynamic[$const_pos][16];
@thigh_marker2=($coorddynamic[$const_pos][17],$coorddynamic[$const_pos][18],$coorddynamic[$const_pos][19]);
$landmarks[5][1]=$coorddynamic[$const_pos][17];
$landmarks[5][2]=$coorddynamic[$const_pos][18];
$landmarks[5][3]=$coorddynamic[$const_pos][19];
@thigh_marker3=($coorddynamic[$const_pos][20],$coorddynamic[$const_pos][21],$coorddynamic[$const_pos][22]);
$landmarks[6][1]=$coorddynamic[$const_pos][20];
$landmarks[6][2]=$coorddynamic[$const_pos][21];
$landmarks[6][3]=$coorddynamic[$const_pos][22];
@shank_marker1=($coorddynamic[$row][23],$coorddynamic[$row][24],$coorddynamic[$row][25]);
$landmarks[7][1]=$coorddynamic[$row][23];
$landmarks[7][2]=$coorddynamic[$row][24];
$landmarks[7][3]=$coorddynamic[$row][25];
@shank_marker2=($coorddynamic[$row][26],$coorddynamic[$row][27],$coorddynamic[$row][28]);
$landmarks[8][1]=$coorddynamic[$row][26];
$landmarks[8][2]=$coorddynamic[$row][27];
$landmarks[8][3]=$coorddynamic[$row][28];
@shank_marker3=($coorddynamic[$row][29],$coorddynamic[$row][30],$coorddynamic[$row][31]);
$landmarks[9][1]=$coorddynamic[$row][29];
$landmarks[9][2]=$coorddynamic[$row][30];
$landmarks[9][3]=$coorddynamic[$row][31];
# static markers - re-generate from relative vectors

# marker coordinate systems
# thigh
$th_mark_coord_center[0]=($thigh_marker2[0]+$thigh_marker3[0])/2;
$th_mark_coord_center[1]=($thigh_marker2[1]+$thigh_marker3[1])/2;
$th_mark_coord_center[2]=($thigh_marker2[2]+$thigh_marker3[2])/2;

# thigh marker coord x
$th_mark_coord_x[0]=$thigh_marker1[0]-$th_mark_coord_center[0];
$th_mark_coord_x[1]=$thigh_marker1[1]-$th_mark_coord_center[1];
$th_mark_coord_x[2]=$thigh_marker1[2]-$th_mark_coord_center[2];

# thigh marker coord z
$th_mark_coord_z[0]=$thigh_marker3[0]-$th_mark_coord_center[0];
$th_mark_coord_z[1]=$thigh_marker3[1]-$th_mark_coord_center[1];
$th_mark_coord_z[2]=$thigh_marker3[2]-$th_mark_coord_center[2];

# normalise x and z
@th_mark_coord_x_norm=&normalise(@th_mark_coord_x);
@th_mark_coord_z_norm=&normalise(@th_mark_coord_z);

# cross product of z and x gives y vector
@th_mark_coord_y=&cross(@th_mark_coord_z_norm,@th_mark_coord_x_norm);
@th_mark_coord_y_norm=&normalise(@th_mark_coord_y);

# cross product of x and y gives new z (overwrite previous z vector)
@th_mark_coord_z=&cross(@th_mark_coord_x_norm,@th_mark_coord_y_norm);
@th_mark_coord_z_norm=&normalise(@th_mark_coord_z);

# shank
$sh_mark_coord_center[0]=($shank_marker2[0]+$shank_marker3[0])/2;
$sh_mark_coord_center[1]=($shank_marker2[1]+$shank_marker3[1])/2;
$sh_mark_coord_center[2]=($shank_marker2[2]+$shank_marker3[2])/2;

# thigh marker coord x
$sh_mark_coord_x[0]=$shank_marker1[0]-$sh_mark_coord_center[0];
$sh_mark_coord_x[1]=$shank_marker1[1]-$sh_mark_coord_center[1];
$sh_mark_coord_x[2]=$shank_marker1[2]-$sh_mark_coord_center[2];

# thigh marker coord z
$sh_mark_coord_z[0]=$shank_marker3[0]-$sh_mark_coord_center[0];
$sh_mark_coord_z[1]=$shank_marker3[1]-$sh_mark_coord_center[1];
$sh_mark_coord_z[2]=$shank_marker3[2]-$sh_mark_coord_center[2];

# normalise x and z
@sh_mark_coord_x_norm=&normalise(@sh_mark_coord_x);
@sh_mark_coord_z_norm=&normalise(@sh_mark_coord_z);

# cross product of z and x gives y vector
@sh_mark_coord_y=&cross(@sh_mark_coord_z_norm,@sh_mark_coord_x_norm);
@sh_mark_coord_y_norm=&normalise(@sh_mark_coord_y);

# cross product of x and y gives new z (overwrite previous z vector)
@sh_mark_coord_z=&cross(@sh_mark_coord_x_norm,@sh_mark_coord_y_norm);
@sh_mark_coord_z_norm=&normalise(@sh_mark_coord_z);

# regenerate condyles for knee
@rot_matrix=&rot_markertolab(@lab_x_norm,@lab_y_norm,@lab_z_norm,@th_mark_coord_x_norm,@th_mark_coord_y_norm,@th_mark_coord_z_norm);

$origin_rel[0]=$th_mark_coord_center[0];
$origin_rel[1]=$th_mark_coord_center[1];
$origin_rel[2]=$th_mark_coord_center[2];

@lat_knee=&generate_condyle_lab(@rot_matrix,@lat_knee_rel,@origin_rel);
$landmarks[10][1]=$lat_knee[0];
$landmarks[10][2]=$lat_knee[1];
$landmarks[10][3]=$lat_knee[2];
@med_knee=&generate_condyle_lab(@rot_matrix,@med_knee_rel,@origin_rel);
$landmarks[11][1]=$med_knee[0];
$landmarks[11][2]=$med_knee[1];
$landmarks[11][3]=$med_knee[2];

# regenerate condyles for ankle
@rot_matrix=&rot_markertolab(@lab_x_norm,@lab_y_norm,@lab_z_norm,@sh_mark_coord_x_norm,@sh_mark_coord_y_norm,@sh_mark_coord_z_norm);

$origin_rel[0]=$sh_mark_coord_center[0];
$origin_rel[1]=$sh_mark_coord_center[1];
$origin_rel[2]=$sh_mark_coord_center[2];

@lat_ankle=&generate_condyle_lab(@rot_matrix,@lat_ankle_rel,@origin_rel);
$landmarks[12][1]=$lat_ankle[0];
$landmarks[12][2]=$lat_ankle[1];
$landmarks[12][3]=$lat_ankle[2];
@med_ankle=&generate_condyle_lab(@rot_matrix,@med_ankle_rel,@origin_rel);
$landmarks[13][1]=$med_ankle[0];
$landmarks[13][2]=$med_ankle[1];
$landmarks[13][3]=$med_ankle[2];

# temp: regenerate knee center with respect to shank marker system
#@shank_knee_center=&generate_condyle_lab(@rot_matrix,@shank_knee_center_rel,@origin_rel);
#$landmarks[18][1]=$shank_knee_center[0];
#$landmarks[18][2]=$shank_knee_center[1];
#$landmarks[18][3]=$shank_knee_center[2];

$landmarks[18][1]=$shank_knee_center[0];
$landmarks[18][2]=$shank_knee_center[1];
$landmarks[18][3]=$shank_knee_center[2];

# generated positions
$hip_center[0]=($l_asis[0]+$r_asis[0])/2;
$hip_center[1]=($l_asis[1]+$r_asis[1])/2;
$hip_center[2]=($l_asis[2]+$r_asis[2])/2;
$landmarks[14][1]=$hip_center[0];
$landmarks[14][2]=$hip_center[1];
$landmarks[14][3]=$hip_center[2];
$knee_center[0]=($lat_knee[0]+$med_knee[0])/2;
$knee_center[1]=($lat_knee[1]+$med_knee[1])/2;
$knee_center[2]=($lat_knee[2]+$med_knee[2])/2;
$landmarks[15][1]=$knee_center[0];
$landmarks[15][2]=$knee_center[1];
$landmarks[15][3]=$knee_center[2];
$ankle_center[0]=($lat_ankle[0]+$med_ankle[0])/2;
$ankle_center[1]=($lat_ankle[1]+$med_ankle[1])/2;
$ankle_center[2]=($lat_ankle[2]+$med_ankle[2])/2;
$landmarks[16][1]=$ankle_center[0];
$landmarks[16][2]=$ankle_center[1];
$landmarks[16][3]=$ankle_center[2];

# work out pelvic coordinate system
$hip_y[0]=$l_asis[0]-$hip_center[0];
$hip_y[1]=$l_asis[1]-$hip_center[1];
$hip_y[2]=$l_asis[2]-$hip_center[2];

$hip_x[0]=$v_sacral[0]-$hip_center[0];
$hip_x[1]=$v_sacral[1]-$hip_center[1];
$hip_x[2]=$v_sacral[2]-$hip_center[2];

# normalise x and y
@hip_y_norm=&normalise(@hip_y);
@hip_x_norm=&normalise(@hip_x);

# cross product of x and y gives z vector
@hip_z=&cross(@hip_x_norm,@hip_y_norm);
@hip_z_norm=&normalise(@hip_z);

# cross product of y and z gives new x (overwrite previous x vector)
@hip_x=&cross(@hip_y_norm,@hip_z_norm);
@hip_x_norm=&normalise(@hip_x);

# Assis distance
$assis_d = Math::BigFloat->new(($l_asis[0]-$r_asis[0])**2+($l_asis[1]-$r_asis[1])**2+($l_asis[2]-$r_asis[2])**2);
$assis_d=$assis_d->fsqrt(3);

# Work out femur ball center
$xd=0.22*$assis_d;
$yd=0.32*$assis_d;
$zd=0.34*$assis_d;
$femur_ball[0]=$hip_center[0]+($xd*$hip_x_norm[0])+($yd*$hip_y_norm[0])+($zd*$hip_z_norm[0]);
$femur_ball[1]=$hip_center[1]+($xd*$hip_x_norm[1])+($yd*$hip_y_norm[1])+($zd*$hip_z_norm[1]);
$femur_ball[2]=$hip_center[2]+($xd*$hip_x_norm[2])+($yd*$hip_y_norm[2])+($zd*$hip_z_norm[2]);
$landmarks[17][1]=$femur_ball[0];
$landmarks[17][2]=$femur_ball[1];
$landmarks[17][3]=$femur_ball[2];

# work out thigh coordinate system
# uses femur ball center.
$thigh_x[0]=$femur_ball[0]-$knee_center[0];
$thigh_x[1]=$femur_ball[1]-$knee_center[1];
$thigh_x[2]=$femur_ball[2]-$knee_center[2];

$thigh_z[0]=$lat_knee[0]-$knee_center[0];
$thigh_z[1]=$lat_knee[1]-$knee_center[1];
$thigh_z[2]=$lat_knee[2]-$knee_center[2];

# normalise x and z
@thigh_x_norm=&normalise(@thigh_x);
@thigh_z_norm=&normalise(@thigh_z);

# cross product of z and x gives y vector
@thigh_y=&cross(@thigh_z_norm,@thigh_x_norm);
@thigh_y_norm=&normalise(@thigh_y);

# cross product of x and y gives new z (overwrite previous z vector)
@thigh_z=&cross(@thigh_x_norm,@thigh_y_norm);
@thigh_z_norm=&normalise(@thigh_z);

# work out shank coordinate system
# Create new shank centre from moving ICR
#$shank_knee_center[0]=$knee_center[0];
#$shank_knee_center[1]=$knee_center[1]-20*cos($q*3.141/180)-20;
#$shank_knee_center[2]=$knee_center[2]+30*sin($q*3.141/180);
#$q=$q+0.8;

# Temp : Use shank_knee_center rather than knee_center relative to thigh 
#$shank_x[0]=$shank_knee_center[0]-$ankle_center[0];
#$shank_x[1]=$shank_knee_center[1]-$ankle_center[1];
#$shank_x[2]=$shank_knee_center[2]-$ankle_center[2];
# uses knee center.
$shank_x[0]=$knee_center[0]-$ankle_center[0];
$shank_x[1]=$knee_center[1]-$ankle_center[1];
$shank_x[2]=$knee_center[2]-$ankle_center[2];

$shank_z[0]=$lat_ankle[0]-$ankle_center[0];
$shank_z[1]=$lat_ankle[1]-$ankle_center[1];
$shank_z[2]=$lat_ankle[2]-$ankle_center[2];

# normalise x and z
@shank_x_norm=&normalise(@shank_x);
@shank_z_norm=&normalise(@shank_z);

# cross product of z and x gives y vector
@shank_y=&cross(@shank_z_norm,@shank_x_norm);
@shank_y_norm=&normalise(@shank_y);

# cross product of x and y gives new z (overwrite previous z vector)
@shank_z=&cross(@shank_x_norm,@shank_y_norm);
@shank_z_norm=&normalise(@shank_z);

# Define joint coordinate system (JCS) unit vectors
# Fixed body axis to femur z axis (e1)
@e1=@thigh_z_norm;
# Fixed body axis to tibia x axis (e3)
@e3=@shank_x_norm;
# Relative axis to e1 (e1_r) directed anteriously.
@e1_r=@thigh_y_norm;
# Relative axis to e2 (e2_r) directed anteriously.
@e3_r=@shank_y_norm;

# Floating axis perpendicular to e1 and e3 (e2)
@e2=&cross(@e3,@e1);
@e2=&normalise(@e2);

# Work out clinical translations.
# Take vector that joins origins of femur and shank local coordinate systems (Tr).
# This vector is with respect to laboratory reference frame.
# Project this onto the jcs vectors (e1,e2,e3) q1=Tr.e1, q2=Tr.e2, q3=Tr.e3 to give clinical translations.

$Tr[0]=$ankle_center[0]-$knee_center[0];
$Tr[1]=$ankle_center[1]-$knee_center[1];
$Tr[2]=$ankle_center[2]-$knee_center[2];

$q[0]=&dot(@Tr,@e1);
$q[1]=&dot(@Tr,@e2);
$q[2]=&dot(@Tr,@e3);

# Print out knee flexion angles
if ($k==1){print output_text2 "flexion_extension" . "  " . "axial_rotation" . "  " . "abduction_adduction" . "  " . "q1" . "  " . "q2" . "  " . "q3". "\n";}
print output_text2 $k . "  ";
$flex_ext_angle=&flexion_extension(@e2,@e1_r);
print output_text2 (180-($flex_ext_angle*180/pi)) . "  ";
$axi_rot_angle=&axial_rotation(@e2,@e3_r);
print output_text2 (180-($axi_rot_angle*180/pi)) . "  ";
$abd_add_angle=&abduction_adduction(@e1,@e3);
print output_text2 (90-($abd_add_angle*180/pi)) . "  ";
print output_text2 $q[0] . "  " . $q[1] . "  " . $q[2] . "\n";

# print out gnu files 
print output_gnu_flex $k . "  " . (180-($flex_ext_angle*180/pi)) . "\n";
print output_gnu_rot $k . "  " . (180-($axi_rot_angle*180/pi)). "\n";
print output_gnu_abd $k . "  " . (90-($abd_add_angle*180/pi)). "\n";

# print out files for cmiss
# Need to write out angles,3 axis for rotation (e1,e2,e3) and position to rotate about (ie center of local coordinate systems)
# angles in text file
print output_cmiss_angles (180-($flex_ext_angle*180/pi)) . " ";
print output_cmiss_angles (180-($axi_rot_angle*180/pi)) . " ";
print output_cmiss_angles (90-($abd_add_angle*180/pi)) . "\n";


# Work out rotation matrix for euclidean transformation
# find rotation for tibia (output_cmiss_rot_tra)

# First we rotate the points to their new position
# for flexion1
# rotate by -2 axis 1,0,0
# rotate by 6 axis 0,1,0
# rotate 97 axis 0,0,1
# translate by 394,-965,-401
# for flexion9
# rotate by -1 axis 1,0,0
# rotate by 5 axis 0,1,0
# rotate 97 axis 0,0,1
# translate by 349,-817,-436
# new transformation for 9
# rotate by -1 axis 1,0,0
# rotate by 5 axis 0,1,0
# rotate 76 axis 0,0,1
# translate by 35,-866,-436


$xpi=-1*pi/180;
$ypi=5*pi/180;
$zpi=97*pi/180;

$r11=cos($ypi)*cos($zpi);
$r12=sin($xpi)*sin($ypi)*cos($zpi)-cos($xpi)*sin($zpi);
$r13=cos($xpi)*sin($ypi)*cos($zpi)+sin($xpi)*sin($zpi);
$r21=cos($ypi)*sin($zpi);
$r22=sin($xpi)*sin($ypi)*sin($zpi)+cos($xpi)*cos($zpi);
$r23=cos($xpi)*sin($ypi)*sin($zpi)-sin($xpi)*cos($zpi);
$r31=-sin($ypi);
$r32=sin($xpi)*cos($ypi);
$r33=cos($xpi)*cos($ypi);

$Rot = new Math::Matrix ([$r11,$r12,$r13,349],
                         [$r21,$r22,$r23,-817],
                         [$r31,$r32,$r33,-436],
                         [0,0,0,1]);
		      

# shank marker system points current

# Only go through first time 
if ($z==90)
{
  for $c (0..2)
  {
   $x1[$c]=$ankle_center[$c];
   $x2[$c]=$shank_x_norm[$c]+$ankle_center[$c];
   $x3[$c]=$shank_y_norm[$c]+$ankle_center[$c]; 
   $x4[$c]=$shank_z_norm[$c]+$ankle_center[$c];  
  }
  
$x1ta = new Math::Matrix ([$x1[0],$x1[1],$x1[2],1]);
$x2ta = new Math::Matrix ([$x2[0],$x2[1],$x2[2],1]);
$x3ta = new Math::Matrix ([$x3[0],$x3[1],$x3[2],1]);
$x4ta = new Math::Matrix ([$x4[0],$x4[1],$x4[2],1]);


$x1tb=$x1ta->transpose;
$x2tb=$x2ta->transpose;
$x3tb=$x3ta->transpose;
$x4tb=$x4ta->transpose;

$x1tc=$Rot->multiply($x1tb);
@x1tc = split(/\s+/,$x1tc);
$x2tc=$Rot->multiply($x2tb);
@x2tc = split(/\s+/,$x2tc); 
$x3tc=$Rot->multiply($x3tb);
@x3tc = split(/\s+/,$x3tc); 
$x4tc=$Rot->multiply($x4tb);
@x4tc = split(/\s+/,$x4tc); 

}

# Don't go through first time 
if ($z>90)
{
  for $c (0..2)
  {

     $x1d[$c]=$ankle_center[$c];
     $x2d[$c]=$shank_x_norm[$c]+$ankle_center[$c];
     $x3d[$c]=$shank_y_norm[$c]+$ankle_center[$c]; 
     $x4d[$c]=$shank_z_norm[$c]+$ankle_center[$c];      
    
     $x1dta = new Math::Matrix ([$x1d[0],$x1d[1],$x1d[2],1]);
     $x2dta = new Math::Matrix ([$x2d[0],$x2d[1],$x2d[2],1]);
     $x3dta = new Math::Matrix ([$x3d[0],$x3d[1],$x3d[2],1]);
     $x4dta = new Math::Matrix ([$x4d[0],$x4d[1],$x4d[2],1]); 
    
     $x1dtb=$x1dta->transpose;
     $x2dtb=$x2dta->transpose;
     $x3dtb=$x3dta->transpose;
     $x4dtb=$x4dta->transpose;  

     $x1dtc=$Rot->multiply($x1dtb);
     @x1dtc = split(/\s+/,$x1dtc);       
     $x2dtc=$Rot->multiply($x2dtb);
     @x2dtc = split(/\s+/,$x2dtc);      
     $x3dtc=$Rot->multiply($x3dtb);
     @x3dtc = split(/\s+/,$x3dtc); 
     $x4dtc=$Rot->multiply($x4dtb);
     @x4dtc = split(/\s+/,$x4dtc);           
     
  } 
  
#  print $x1tc[1] .  " " . $x1tc[2] . " " . $x1tc[3] . "\n";
#  print $x2tc[1] .  " " . $x2tc[2] . " " . $x2tc[3] . "\n";  
#  print $x3tc[1] .  " " . $x3tc[2] . " " . $x3tc[3] . "\n";
#  print $x4tc[1] .  " " . $x4tc[2] . " " . $x4tc[3] . "\n";
#  print "\n";
#  print $x1dtc[1] .  " " . $x1dtc[2] . " " . $x1dtc[3] . "\n";
#  print $x2dtc[1] .  " " . $x2dtc[2] . " " . $x2dtc[3] . "\n";  
#  print $x3dtc[1] .  " " . $x3dtc[2] . " " . $x3dtc[3] . "\n";
#  print $x4dtc[1] .  " " . $x4dtc[2] . " " . $x4dtc[3] . "\n";
#  print "\n";  
#  print "solve" . "\n";
  
  


  $a = new Math::Matrix ([$x1tc[1],$x1tc[2],$x1tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],
                         [0,0,0,0,$x1tc[1],$x1tc[2],$x1tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$x1tc[1],$x1tc[2],$x1tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$x1tc[1],$x1tc[2],$x1tc[3],1],
                         [$x2tc[1],$x2tc[2],$x2tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$x2tc[1],$x2tc[2],$x2tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$x2tc[1],$x2tc[2],$x2tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$x2tc[1],$x2tc[2],$x2tc[3],1],		       
                         [$x3tc[1],$x3tc[2],$x3tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$x3tc[1],$x3tc[2],$x3tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$x3tc[1],$x3tc[2],$x3tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$x3tc[1],$x3tc[2],$x3tc[3],1],
                         [$x4tc[1],$x4tc[2],$x4tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$x4tc[1],$x4tc[2],$x4tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$x4tc[1],$x4tc[2],$x4tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$x4tc[1],$x4tc[2],$x4tc[3],1]);		       
  $x = new Math::Matrix ([$x1dtc[1],$x1dtc[2],$x1dtc[3],1,$x2dtc[1],$x2dtc[2],$x2dtc[3],1,$x3dtc[1],$x3dtc[2],$x3dtc[3],1,$x4dtc[1],$x4dtc[2],$x4dtc[3],1]);
#  $a->print("A\n");
  $E = $a->concat($x->transpose);
#  $E->print("Equation system\n");
  $s = $E->solve;
#  $s->print("Solutions s\n");
  @s = split(/\s+/,$s);  
#  $a->multiply($s)->print("A*s\n");

# now store this position as previous for next loop
  for $c (1..3)
  {
   $prev1[$c]=$x1tc[$c];
   $prev2[$c]=$x2tc[$c];
   $prev3[$c]=$x3tc[$c];   
   $prev4[$c]=$x4tc[$c];       
   $x1tc[$c]=$x1dtc[$c];
   $x2tc[$c]=$x2dtc[$c];
   $x3tc[$c]=$x3dtc[$c];
   $x4tc[$c]=$x4dtc[$c];  
  }


#Now work out cardan angles
# Rotation about y-axis (y_angle)
$y_angle1=-asin($s[9]);
$y_angle2=pi-asin($s[9]);

# Rotation about x-axis (x_angle) 
$x_angle1=asin(($s[10]/cos($y_angle1)));
$x_angle2=pi-asin(($s[10]/cos($y_angle1)));
$x_angle3=asin(($s[10]/cos($y_angle2)));
$x_angle4=pi-asin(($s[10]/cos($y_angle2)));

# Rotation about x-axis (z_angle) 
$z_angle1=asin(($s[5]/cos($y_angle1)));
$z_angle2=pi-asin(($s[5]/cos($y_angle1)));
$z_angle3=asin(($s[5]/cos($y_angle2)));
$z_angle4=pi-asin(($s[5]/cos($y_angle2)));

# This gives 8 combinations (must eliminate sum)
$ang[1][0]=$x_angle1;
$ang[1][1]=$y_angle1;
$ang[1][2]=$z_angle1;

$ang[2][0]=$x_angle1;
$ang[2][1]=$y_angle1;
$ang[2][2]=$z_angle2;

$ang[3][0]=$x_angle2;
$ang[3][1]=$y_angle1;
$ang[3][2]=$z_angle1;

$ang[4][0]=$x_angle2;
$ang[4][1]=$y_angle1;
$ang[4][2]=$z_angle2;

$ang[5][0]=$x_angle3;
$ang[5][1]=$y_angle2;
$ang[5][2]=$z_angle3;

$ang[6][0]=$x_angle3;
$ang[6][1]=$y_angle2;
$ang[6][2]=$z_angle4;

$ang[7][0]=$x_angle4;
$ang[7][1]=$y_angle2;
$ang[7][2]=$z_angle3;

$ang[8][0]=$x_angle4;
$ang[8][1]=$y_angle2;
$ang[8][2]=$z_angle4;

# must satisfy criteria
for $t (1..1)
{
 $pass=1;
 if ( abs($s[1]-(cos($ang[$t][1])*cos($ang[$t][2])))<0.001 ) {
#  print "pass" ."\n";
 }else{
  $pass=0; 
#  print "fail" ."\n";  
 }
 
 if ( abs($s[2]-(sin($ang[$t][0])*sin($ang[$t][1])*cos($ang[$t][2])-cos($ang[$t][0])*sin($ang[$t][2])))<0.001 ) {
#  print "pass" ."\n";  
 }else{
  $pass=0; 
#  print "fail" ."\n";   
 }
 
 if ( abs($s[3]-(cos($ang[$t][0])*sin($ang[$t][1])*cos($ang[$t][2])+sin($ang[$t][0])*sin($ang[$t][2])))<0.001 ) {
#  print "pass" ."\n";    
 }else{
  $pass=0; 
#  print "fail" ."\n";   
 } 
  
 if ( abs($s[6]-(sin($ang[$t][0])*sin($ang[$t][1])*sin($ang[$t][2])+cos($ang[$t][0])*cos($ang[$t][2])))<0.001 ) {
#  print "pass" ."\n";  
 }else{
  $pass=0; 
#  print "fail" ."\n";   
 }   
 
 if ( abs($s[7]-(cos($ang[$t][0])*sin($ang[$t][1])*sin($ang[$t][2])-sin($ang[$t][0])*cos($ang[$t][2])))<0.001 ) {
#  print "pass" ."\n";   
 }else{
  $pass=0; 
#  print "fail" ."\n";   
 }    
 
 if ( abs($s[11]-(cos($ang[$t][0])*cos($ang[$t][1])))<0.001 ) {
#  print "pass" ."\n";   
 }else{
  $pass=0;
#  print "fail" ."\n";    
 } 
  
 if ($pass==1) {
  $correct_angles[0]=$ang[$t][0];
  $correct_angles[1]=$ang[$t][1]; 
  $correct_angles[2]=$ang[$t][2]; 
  }
  
}

$translation[0]=$s[4];
$translation[1]=$s[8];
$translation[2]=$s[12];

print output_cmiss_rot_tra $correct_angles[0]*180/pi ." ";
print output_cmiss_rot_tra $correct_angles[1]*180/pi ." ";
print output_cmiss_rot_tra $correct_angles[2]*180/pi ." ";
print output_cmiss_rot_tra $translation[0] ." ";
print output_cmiss_rot_tra $translation[1] ." ";
print output_cmiss_rot_tra $translation[2] ." ";

# check if values are correct
$xpi=$correct_angles[0];
$ypi=$correct_angles[1];
$zpi=$correct_angles[2];

$r11=cos($ypi)*cos($zpi);
$r12=sin($xpi)*sin($ypi)*cos($zpi)-cos($xpi)*sin($zpi);
$r13=cos($xpi)*sin($ypi)*cos($zpi)+sin($xpi)*sin($zpi);
$r21=cos($ypi)*sin($zpi);
$r22=sin($xpi)*sin($ypi)*sin($zpi)+cos($xpi)*cos($zpi);
$r23=cos($xpi)*sin($ypi)*sin($zpi)-sin($xpi)*cos($zpi);
$r31=-sin($ypi);
$r32=sin($xpi)*cos($ypi);
$r33=cos($xpi)*cos($ypi);

$Rot = new Math::Matrix ([$r11,$r12,$r13,$translation[0]],
                         [$r21,$r22,$r23,$translation[1]],
                         [$r31,$r32,$r33,$translation[2]],
                         [0,0,0,1]);
			 
			 
$x1dta = new Math::Matrix ([$prev1[1],$prev1[2],$prev1[3],1]);
$x2dta = new Math::Matrix ([$prev2[1],$prev2[2],$prev2[3],1]);
$x3dta = new Math::Matrix ([$prev3[1],$prev3[2],$prev3[3],1]);
$x4dta = new Math::Matrix ([$prev4[1],$prev4[2],$prev4[3],1]); 
    
$x1dtb=$x1dta->transpose;
$x2dtb=$x2dta->transpose;
$x3dtb=$x3dta->transpose;
$x4dtb=$x4dta->transpose;  

$x1dtc=$Rot->multiply($x1dtb);
@x1dtc = split(/\s+/,$x1dtc);       
$x2dtc=$Rot->multiply($x2dtb);
@x2dtc = split(/\s+/,$x2dtc);      
$x3dtc=$Rot->multiply($x3dtb);
@x3dtc = split(/\s+/,$x3dtc); 
$x4dtc=$Rot->multiply($x4dtb);
@x4dtc = split(/\s+/,$x4dtc);

#print "re-generated" . "\n";
#print $x1dtc[1] .  " " . $x1dtc[2] . " " . $x1dtc[3] . "\n";
#print $x2dtc[1] .  " " . $x2dtc[2] . " " . $x2dtc[3] . "\n";  
#print $x3dtc[1] .  " " . $x3dtc[2] . " " . $x3dtc[3] . "\n";
#print $x4dtc[1] .  " " . $x4dtc[2] . " " . $x4dtc[3] . "\n";
#print "\n";  


}
# $z>80


# Work out rotation matrix for euclidean transformation
# find rotation for femur (output_cmiss_rot_tra)

# First we rotate the points to their new position
# for flexion1
# rotate by -2 axis 1,0,0
# rotate by 6 axis 0,1,0
# rotate 97 axis 0,0,1
# translate by 394,-965,-401
# for flexion9
# rotate by -1 axis 1,0,0
# rotate by 5 axis 0,1,0
# rotate 97 axis 0,0,1
# translate by 349,-817,-436
# new transformation for 9
# rotate by -1 axis 1,0,0
# rotate by 5 axis 0,1,0
# rotate 76 axis 0,0,1
# translate by 35,-866,-436


$xpi=-1*pi/180;
$ypi=5*pi/180;
$zpi=97*pi/180;

$r11=cos($ypi)*cos($zpi);
$r12=sin($xpi)*sin($ypi)*cos($zpi)-cos($xpi)*sin($zpi);
$r13=cos($xpi)*sin($ypi)*cos($zpi)+sin($xpi)*sin($zpi);
$r21=cos($ypi)*sin($zpi);
$r22=sin($xpi)*sin($ypi)*sin($zpi)+cos($xpi)*cos($zpi);
$r23=cos($xpi)*sin($ypi)*sin($zpi)-sin($xpi)*cos($zpi);
$r31=-sin($ypi);
$r32=sin($xpi)*cos($ypi);
$r33=cos($xpi)*cos($ypi);

$Rot = new Math::Matrix ([$r11,$r12,$r13,349],
                         [$r21,$r22,$r23,-817],
                         [$r31,$r32,$r33,-436],
                         [0,0,0,1]);
		      

# femur marker system points current

# Only go through first time 
if ($z==90)
{
  for $thc (0..2)
  {
   $thx1[$thc]=$knee_center[$thc];
   $thx2[$thc]=$thigh_x_norm[$thc]+$knee_center[$thc];
   $thx3[$thc]=$thigh_y_norm[$thc]+$knee_center[$thc]; 
   $thx4[$thc]=$thigh_z_norm[$thc]+$knee_center[$thc];  
  }
  
$thx1ta = new Math::Matrix ([$thx1[0],$thx1[1],$thx1[2],1]);
$thx2ta = new Math::Matrix ([$thx2[0],$thx2[1],$thx2[2],1]);
$thx3ta = new Math::Matrix ([$thx3[0],$thx3[1],$thx3[2],1]);
$thx4ta = new Math::Matrix ([$thx4[0],$thx4[1],$thx4[2],1]);


$thx1tb=$thx1ta->transpose;
$thx2tb=$thx2ta->transpose;
$thx3tb=$thx3ta->transpose;
$thx4tb=$thx4ta->transpose;

$thx1tc=$Rot->multiply($thx1tb);
@thx1tc = split(/\s+/,$thx1tc);
$thx2tc=$Rot->multiply($thx2tb);
@thx2tc = split(/\s+/,$thx2tc); 
$thx3tc=$Rot->multiply($thx3tb);
@thx3tc = split(/\s+/,$thx3tc); 
$thx4tc=$Rot->multiply($thx4tb);
@thx4tc = split(/\s+/,$thx4tc);

}

# Don't go through first time 
if ($z>90)
{
  for $thc (0..2)
  {
     $thx1d[$thc]=$knee_center[$thc];
     $thx2d[$thc]=$thigh_x_norm[$thc]+$knee_center[$thc];
     $thx3d[$thc]=$thigh_y_norm[$thc]+$knee_center[$thc]; 
     $thx4d[$thc]=$thigh_z_norm[$thc]+$knee_center[$thc];  
     
     $thx1dta = new Math::Matrix ([$thx1d[0],$thx1d[1],$thx1d[2],1]);
     $thx2dta = new Math::Matrix ([$thx2d[0],$thx2d[1],$thx2d[2],1]);
     $thx3dta = new Math::Matrix ([$thx3d[0],$thx3d[1],$thx3d[2],1]);
     $thx4dta = new Math::Matrix ([$thx4d[0],$thx4d[1],$thx4d[2],1]); 
    
     $thx1dtb=$thx1dta->transpose;
     $thx2dtb=$thx2dta->transpose;
     $thx3dtb=$thx3dta->transpose;
     $thx4dtb=$thx4dta->transpose;  

     $thx1dtc=$Rot->multiply($thx1dtb);
     @thx1dtc = split(/\s+/,$thx1dtc);       
     $thx2dtc=$Rot->multiply($thx2dtb);
     @thx2dtc = split(/\s+/,$thx2dtc);      
     $thx3dtc=$Rot->multiply($thx3dtb);
     @thx3dtc = split(/\s+/,$thx3dtc); 
     $thx4dtc=$Rot->multiply($thx4dtb);
     @thx4dtc = split(/\s+/,$thx4dtc);           

  } 
  
#  print $thx1tc[1] .  " " . $thx1tc[2] . " " . $thx1tc[3] . "\n";
#  print $thx2tc[1] .  " " . $thx2tc[2] . " " . $thx2tc[3] . "\n";  
#  print $thx3tc[1] .  " " . $thx3tc[2] . " " . $thx3tc[3] . "\n";
#  print $thx4tc[1] .  " " . $thx4tc[2] . " " . $thx4tc[3] . "\n";
#  print "\n";
#  print $thx1dtc[1] .  " " . $thx1dtc[2] . " " . $thx1dtc[3] . "\n";
#  print $thx2dtc[1] .  " " . $thx2dtc[2] . " " . $thx2dtc[3] . "\n";  
#  print $thx3dtc[1] .  " " . $thx3dtc[2] . " " . $thx3dtc[3] . "\n";
#  print $thx4dtc[1] .  " " . $thx4dtc[2] . " " . $thx4dtc[3] . "\n";
#  print "\n";  
#  print "solve" . "\n";
  
  


  $tha = new Math::Matrix ([$thx1tc[1],$thx1tc[2],$thx1tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],
                         [0,0,0,0,$thx1tc[1],$thx1tc[2],$thx1tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$thx1tc[1],$thx1tc[2],$thx1tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$thx1tc[1],$thx1tc[2],$thx1tc[3],1],
                         [$thx2tc[1],$thx2tc[2],$thx2tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$thx2tc[1],$thx2tc[2],$thx2tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$thx2tc[1],$thx2tc[2],$thx2tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$thx2tc[1],$thx2tc[2],$thx2tc[3],1],		       
                         [$thx3tc[1],$thx3tc[2],$thx3tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$thx3tc[1],$thx3tc[2],$thx3tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$thx3tc[1],$thx3tc[2],$thx3tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$thx3tc[1],$thx3tc[2],$thx3tc[3],1],
                         [$thx4tc[1],$thx4tc[2],$thx4tc[3],1,0,0,0,0,0,0,0,0,0,0,0,0],		       
                         [0,0,0,0,$thx4tc[1],$thx4tc[2],$thx4tc[3],1,0,0,0,0,0,0,0,0],
                         [0,0,0,0,0,0,0,0,$thx4tc[1],$thx4tc[2],$thx4tc[3],1,0,0,0,0],
                         [0,0,0,0,0,0,0,0,0,0,0,0,$thx4tc[1],$thx4tc[2],$thx4tc[3],1]);		       
  $thx = new Math::Matrix ([$thx1dtc[1],$thx1dtc[2],$thx1dtc[3],1,$thx2dtc[1],$thx2dtc[2],$thx2dtc[3],1,$thx3dtc[1],$thx3dtc[2],$thx3dtc[3],1,$thx4dtc[1],$thx4dtc[2],$thx4dtc[3],1]);
#  $tha->print("A\n");
  $thE = $tha->concat($thx->transpose);
#  $thE->print("Equation system\n");
  $ths = $thE->solve;
#  $ths->print("Solutions s\n");
  @ths = split(/\s+/,$ths);  
#  $tha->multiply($ths)->print("A*s\n");

# now store this position as previous for next loop
  for $thc (1..3)
  {
   $thprev1[$thc]=$thx1tc[$thc];
   $thprev2[$thc]=$thx2tc[$thc];
   $thprev3[$thc]=$thx3tc[$thc];   
   $thprev4[$thc]=$thx4tc[$thc];       
   $thx1tc[$thc]=$thx1dtc[$thc];
   $thx2tc[$thc]=$thx2dtc[$thc];
   $thx3tc[$thc]=$thx3dtc[$thc];
   $thx4tc[$thc]=$thx4dtc[$thc];  
  }


#Now work out cardan angles
# Rotation about y-axis (y_angle)
$thy_angle1=-asin($ths[9]);
$thy_angle2=pi-asin($ths[9]);

# Rotation about x-axis (x_angle) 
$thx_angle1=asin(($ths[10]/cos($thy_angle1)));
$thx_angle2=pi-asin(($ths[10]/cos($thy_angle1)));
$thx_angle3=asin(($ths[10]/cos($thy_angle2)));
$thx_angle4=pi-asin(($ths[10]/cos($thy_angle2)));

# Rotation about x-axis (z_angle) 
$thz_angle1=asin(($ths[5]/cos($thy_angle1)));
$thz_angle2=pi-asin(($ths[5]/cos($thy_angle1)));
$thz_angle3=asin(($ths[5]/cos($thy_angle2)));
$thz_angle4=pi-asin(($ths[5]/cos($thy_angle2)));

# This gives 8 combinations (must eliminate sum)
$thang[1][0]=$thx_angle1;
$thang[1][1]=$thy_angle1;
$thang[1][2]=$thz_angle1;

$thang[2][0]=$thx_angle1;
$thang[2][1]=$thy_angle1;
$thang[2][2]=$thz_angle2;

$thang[3][0]=$thx_angle2;
$thang[3][1]=$thy_angle1;
$thang[3][2]=$thz_angle1;

$thang[4][0]=$thx_angle2;
$thang[4][1]=$thy_angle1;
$thang[4][2]=$thz_angle2;

$thang[5][0]=$thx_angle3;
$thang[5][1]=$thy_angle2;
$thang[5][2]=$thz_angle3;

$thang[6][0]=$thx_angle3;
$thang[6][1]=$thy_angle2;
$thang[6][2]=$thz_angle4;

$thang[7][0]=$thx_angle4;
$thang[7][1]=$thy_angle2;
$thang[7][2]=$thz_angle3;

$thang[8][0]=$thx_angle4;
$thang[8][1]=$thy_angle2;
$thang[8][2]=$thz_angle4;

# must satisfy criteria
for $tht (1..1)
{
 $thpass=1;
 if ( abs($ths[1]-(cos($thang[$tht][1])*cos($thang[$tht][2])))<0.001 ) {
#  print "pass" ."\n";
 }else{
  $thpass=0; 
#  print "fail" ."\n";  
 }
 
 if ( abs($ths[2]-(sin($thang[$tht][0])*sin($thang[$tht][1])*cos($thang[$tht][2])-cos($thang[$tht][0])*sin($thang[$tht][2])))<0.001 ) {
#  print "pass" ."\n";  
 }else{
  $thpass=0; 
#  print "fail" ."\n";   
 }
 
 if ( abs($ths[3]-(cos($thang[$tht][0])*sin($thang[$tht][1])*cos($thang[$tht][2])+sin($thang[$tht][0])*sin($thang[$tht][2])))<0.001 ) {
#  print "pass" ."\n";    
 }else{
  $thpass=0; 
#  print "fail" ."\n";   
 } 
  
 if ( abs($ths[6]-(sin($thang[$tht][0])*sin($thang[$tht][1])*sin($thang[$tht][2])+cos($thang[$tht][0])*cos($thang[$tht][2])))<0.001 ) {
#  print "pass" ."\n";  
 }else{
  $thpass=0; 
#  print "fail" ."\n";   
 }   
 
 if ( abs($ths[7]-(cos($thang[$tht][0])*sin($thang[$tht][1])*sin($thang[$tht][2])-sin($thang[$tht][0])*cos($thang[$tht][2])))<0.001 ) {
#  print "pass" ."\n";   
 }else{
  $thpass=0; 
#  print "fail" ."\n";   
 }    
 
 if ( abs($ths[11]-(cos($thang[$tht][0])*cos($thang[$tht][1])))<0.001 ) {
#  print "pass" ."\n";   
 }else{
  $thpass=0;
#  print "fail" ."\n";    
 } 
  
 if ($thpass==1) {
  $thcorrect_angles[0]=$thang[$tht][0];
  $thcorrect_angles[1]=$thang[$tht][1]; 
  $thcorrect_angles[2]=$thang[$tht][2]; 
  }
  
}

$thtranslation[0]=$ths[4];
$thtranslation[1]=$ths[8];
$thtranslation[2]=$ths[12];

print output_cmiss_rot_tra $thcorrect_angles[0]*180/pi ." ";
print output_cmiss_rot_tra $thcorrect_angles[1]*180/pi ." ";
print output_cmiss_rot_tra $thcorrect_angles[2]*180/pi ." ";
print output_cmiss_rot_tra $thtranslation[0] ." ";
print output_cmiss_rot_tra $thtranslation[1] ." ";
print output_cmiss_rot_tra $thtranslation[2] ."\n";

# check if values are correct
$thxpi=$thcorrect_angles[0];
$thypi=$thcorrect_angles[1];
$thzpi=$thcorrect_angles[2];

$thr11=cos($thypi)*cos($thzpi);
$thr12=sin($thxpi)*sin($thypi)*cos($thzpi)-cos($thxpi)*sin($thzpi);
$thr13=cos($thxpi)*sin($thypi)*cos($thzpi)+sin($thxpi)*sin($thzpi);
$thr21=cos($thypi)*sin($thzpi);
$thr22=sin($thxpi)*sin($thypi)*sin($thzpi)+cos($thxpi)*cos($thzpi);
$thr23=cos($thxpi)*sin($thypi)*sin($thzpi)-sin($thxpi)*cos($thzpi);
$thr31=-sin($thypi);
$thr32=sin($thxpi)*cos($thypi);
$thr33=cos($thxpi)*cos($thypi);

$thRot = new Math::Matrix ([$thr11,$thr12,$thr13,$thtranslation[0]],
                         [$thr21,$thr22,$thr23,$thtranslation[1]],
                         [$thr31,$thr32,$thr33,$thtranslation[2]],
                         [0,0,0,1]);
			 
			 
$thx1dta = new Math::Matrix ([$thprev1[1],$thprev1[2],$thprev1[3],1]);
$thx2dta = new Math::Matrix ([$thprev2[1],$thprev2[2],$thprev2[3],1]);
$thx3dta = new Math::Matrix ([$thprev3[1],$thprev3[2],$thprev3[3],1]);
$thx4dta = new Math::Matrix ([$thprev4[1],$thprev4[2],$thprev4[3],1]); 
    
$thx1dtb=$thx1dta->transpose;
$thx2dtb=$thx2dta->transpose;
$thx3dtb=$thx3dta->transpose;
$thx4dtb=$thx4dta->transpose;  

$thx1dtc=$thRot->multiply($thx1dtb);
@thx1dtc = split(/\s+/,$thx1dtc);       
$thx2dtc=$thRot->multiply($thx2dtb);
@thx2dtc = split(/\s+/,$thx2dtc);      
$thx3dtc=$thRot->multiply($thx3dtb);
@thx3dtc = split(/\s+/,$thx3dtc); 
$thx4dtc=$thRot->multiply($thx4dtb);
@thx4dtc = split(/\s+/,$thx4dtc);

#print "re-generated" . "\n";
#print $thx1dtc[1] .  " " . $thx1dtc[2] . " " . $thx1dtc[3] . "\n";
#print $thx2dtc[1] .  " " . $thx2dtc[2] . " " . $thx2dtc[3] . "\n";  
#print $thx3dtc[1] .  " " . $thx3dtc[2] . " " . $thx3dtc[3] . "\n";
#print $thx4dtc[1] .  " " . $thx4dtc[2] . " " . $thx4dtc[3] . "\n";
#print "\n";  


}
# $z>80


##create dynamic exdata file
#print output_exdata3 " Group name: dynamic" . "\n";
#print output_exdata3 " #Fields=1" . "\n";
#print output_exdata3 " 1) coordinates, coordinate, rectangular cartesian, #Components=3" . "\n";
#print output_exdata3 "  x.  Value index=1, #Derivatives=0, #Versions=1" . "\n";
#print output_exdata3 "  y.  Value index=2, #Derivatives=0, #Versions=1" . "\n";
#print output_exdata3 "  z.  Value index=3, #Derivatives=0, #Versions=1" . "\n";
#for $i (1..17)
#{
#print output_exdata3 " Node: " . ($i+50) . "\n";
#print output_exdata3 " " . $landmarks[$i][1] . "\n";
#print output_exdata3 " " . $landmarks[$i][2] . "\n";
#print output_exdata3 " " . $landmarks[$i][3] . "\n";
#}

#create dynamic exdata file with coordinate systems vectors
#print output_exdata4 " Group name: dynamic_vector" . "\n";
#print output_exdata4 " #Fields=2" . "\n";
#print output_exdata4 " 1) coordinates, coordinate, rectangular cartesian, #Components=3" . "\n";
#print output_exdata4 "  x.  Value index=1, #Derivatives=0" . "\n";
#print output_exdata4 "  y.  Value index=2, #Derivatives=0" . "\n";
#print output_exdata4 "  z.  Value index=3, #Derivatives=0" . "\n";
#print output_exdata4 " 2) error, field, rectangular cartesian, #Components=3" . "\n";
#print output_exdata4 "  x.  Value index= 4, #Derivatives=0" . "\n";
#print output_exdata4 "  y.  Value index= 5, #Derivatives=0" . "\n";
#print output_exdata4 "  z.  Value index= 6, #Derivatives=0" . "\n";
#print output_exdata4  " Node:     201". "\n";
#print output_exdata4   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
#print output_exdata4   $thigh_x_norm[0] . " " . $thigh_x_norm[1] . " " . $thigh_x_norm[2]. "\n";
#print output_exdata4  " Node:     202". "\n";
#print output_exdata4   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
#print output_exdata4   $thigh_y_norm[0] . " " . $thigh_y_norm[1] . " " . $thigh_y_norm[2]. "\n";
#print output_exdata4  " Node:     203". "\n";
#print output_exdata4   $knee_center[0] . " " . $knee_center[1] . " " . $knee_center[2]. "\n";
#print output_exdata4   $thigh_z_norm[0] . " " . $thigh_z_norm[1] . " " . $thigh_z_norm[2]. "\n";
#print output_exdata4  " Node:     204". "\n";
#print output_exdata4   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
#print output_exdata4   $hip_x_norm[0] . " " . $hip_x_norm[1] . " " . $hip_x_norm[2]. "\n";
#print output_exdata4  " Node:     205". "\n";
#print output_exdata4   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
#print output_exdata4   $hip_y_norm[0] . " " . $hip_y_norm[1] . " " . $hip_y_norm[2]. "\n";
#print output_exdata4  " Node:     206". "\n";
#print output_exdata4   $hip_center[0] . " " . $hip_center[1] . " " . $hip_center[2]. "\n";
#print output_exdata4   $hip_z_norm[0] . " " . $hip_z_norm[1] . " " . $hip_z_norm[2]. "\n";
#print output_exdata4  " Node:     207". "\n";
#print output_exdata4   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
#print output_exdata4   $shank_x_norm[0] . " " . $shank_x_norm[1] . " " . $shank_x_norm[2]. "\n";
#print output_exdata4  " Node:     208". "\n";
#print output_exdata4   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
#print output_exdata4   $shank_y_norm[0] . " " . $shank_y_norm[1] . " " . $shank_y_norm[2]. "\n";
#print output_exdata4  " Node:     209". "\n";
#print output_exdata4   $ankle_center[0] . " " . $ankle_center[1] . " " . $ankle_center[2]. "\n";
#print output_exdata4   $shank_z_norm[0] . " " . $shank_z_norm[1] . " " . $shank_z_norm[2]. "\n";
#print output_exdata4  " Node:     210". "\n";
#print output_exdata4   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
#print output_exdata4   $th_mark_coord_x_norm[0] . " " . $th_mark_coord_x_norm[1] . " " . $th_mark_coord_x_norm[2] . "\n";
#print output_exdata4  " Node:     211". "\n";
#print output_exdata4   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
#print output_exdata4   $th_mark_coord_y_norm[0] . " " . $th_mark_coord_y_norm[1] . " " . $th_mark_coord_y_norm[2]. "\n";
#print output_exdata4  " Node:     212". "\n";
#print output_exdata4   $th_mark_coord_center[0] . " " . $th_mark_coord_center[1] . " " . $th_mark_coord_center[2] . "\n";
#print output_exdata4   $th_mark_coord_z_norm[0] . " " . $th_mark_coord_z_norm[1] . " " . $th_mark_coord_z_norm[2] . "\n";
#print output_exdata4  " Node:     213". "\n";
#print output_exdata4   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
#print output_exdata4   $sh_mark_coord_x_norm[0] . " " . $sh_mark_coord_x_norm[1] . " " . $sh_mark_coord_x_norm[2] . "\n";
#print output_exdata4  " Node:     214". "\n";
#print output_exdata4   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
#print output_exdata4   $sh_mark_coord_y_norm[0] . " " . $sh_mark_coord_y_norm[1] . " " . $sh_mark_coord_y_norm[2]. "\n";
#print output_exdata4  " Node:     215". "\n";
#print output_exdata4   $sh_mark_coord_center[0] . " " . $sh_mark_coord_center[1] . " " . $sh_mark_coord_center[2] . "\n";
#print output_exdata4   $sh_mark_coord_z_norm[0] . " " . $sh_mark_coord_z_norm[1] . " " . $sh_mark_coord_z_norm[2] . "\n";
}

# Close the files.
close output_text2;
close output_exdata3;
close output_exdata4;
close output_cmiss_angles;
close output_cmiss_rot_tra;


