# This file read in all the exnode and exelem files for the bladder expansion to create frames on which a movie can be created

#----------------------------
$moviename="bladder_inflation.mv"


#Create some materials
gfx create material bladder_surface ambient 0.68 0 0 diffuse 1 0.74 0.6 emission 0.55 0.05 0. specular 1. 0.2 0 alpha 0.3 shininess 1
gfx create material prostate_surface ambient 1 1 1 diffuse 0 0.26 1 emission 0 0 0 specular 0 0 0 alpha 1 shininess 1
gfx create material ureter_surface ambient 1 0.5 0.1 diffuse 0 0.5 0.1 emission 0 0 0 specular 0 0 0 alpha 1 shininess 1
gfx create material default ambient 1 1 1 diffuse 1 1 1 emission 0 0 0 specular 0 0 0 alpha 1 shininess 0
gfx create material default_selected ambient 1 0 0 diffuse 1 0 0 emission 0 0 0 specular 0 0 0 alpha 1 shininess 0
gfx create material tube ambient 1 1 0.16 diffuse 1 1 0.61 emission 0 0 0 specular 1 1 0.21 alpha 1 shininess 1 
gfx create material translucent_tube ambient 1 1 0.16 diffuse 1 1 0.61 emission 0 0 0 specular 1 1 0.21 alpha 0.5 shininess 1 
gfx create material inner_bladder_surface ambient 0.68 0 0 diffuse 1 0.74 0.6 emission 0.55 0.05 0. specular 1. 0.2 0 alpha 1 shininess 1
gfx cre material fibres ambient 0.5 0 0 diffuse 0.9 0 0 specular 0.5 0.5 0.5 shininess 0.8

# Read in the node and element files.
gfx read node blad_inflate
gfx read element blad_inflate

# Create some element groups
gfx cre egroup ureters add_ranges 51..60,69..78 from undeformed
gfx cre egroup bladder add_ranges 1..50 from undeformed
gfx cre egroup inner_bladder add_ranges 87..104,108..109,111..114,120..125 from undeformed
gfx cre egroup trigone add_ranges 105..107,110,115..119,126..136 from undeformed

# modify the element groups individually.
#gfx modify g_element bladder general clear circle_discretization 2 default_coordinate default_coordinate element_discretization "10*10*10" native_discretization none
#gfx modify g_element ureters general clear circle_discretization 2 default_coordinate default_coordinate element_discretization "10*10*10" native_discretization none
#gfx modify g_element inner_bladder general clear circle_discretization 2 default_coordinate default_coordinate element_discretization "10*10*10" native_discretization none
#gfx modify g_element trigone general clear circle_discretization 2 default_coordinate default_coordinate element_discretization "10*10*10" native_discretization none
#gfx modify g_element undeformed general clear circle_discretization 2 default_coordinate default_coordinate element_discretization "10*10*10" native_discretization none

# Apply surfaces
gfx modify g_element bladder surfaces select_on material bladder_surface face xi3_0 selected_material default_selected render_shaded
gfx modify g_element ureters surfaces select_on material tube selected_material default_selected render_shaded
gfx modify g_element inner_bladder surfaces select_on material inner_bladder_surface face xi3_0 selected_material default_selected render_shaded
gfx modify g_element trigone surfaces select_on material inner_bladder_surface selected_material default_selected render_shaded

gfx modify g_element bladder lines select_on material default selected_material default_selected
gfx modify g_element inner_bladder lines select_on material default face xi3_0 selected_material default_selected
gfx modify g_element ureters lines select_on material default selected_material default_selected
gfx modify g_element trigone lines select_on material default selected_material default_selected

# Set the order of the objects so the inner objects are visible inside the outer objects.
gfx set order object ureters position 1
gfx set order object inner_bladder position 2
gfx set order object trigone position 3
gfx set order object bladder position 4

gfx create window 1
gfx mod win 1 image view_all

# Rotate the view 
gfx mod win 1 image rotate 1 0 0 90
gfx mod win 1 view perspective

# Set the resolution of the window to 400x400
gfx mod win 1 layout width 400 height 400

# Set the interest point, eye point and up vector that define the centre of
# what you are looking at, where you are viewing from and which way is up. CMGUI
# automatically makes sure the up_vector is orthogonal to the direction of view
# - unless you specify the 'allow_skew' options.
gfx mod win 1 view interest_point 0 0 -50 eye_point 0 0 -300 up_vector 0 -1 0

# Set the near and far clipping planes in distance from the view point.
gfx mod win 1 view near_clipping_plane 1 far_clipping_plane 600

# Set the view angle to mostly fill the screen
gfx mod win 1 view view_angle 45
gfx modify window 1 image rotate 0 1 0 120

# Turn on scene antialiasing - this increases rendering time 8-fold, but will
# greatly improve the image quality in the movie, and make it scale better. On
# smaller systems, it is advisable not to turn on antialiasing.
gfx mod win 1 set antialias 8

#Start writing the movie to file 
gfx movie create $moviename rle24_sgi_movie3 window 1

# Add the frame for the initial mesh
gfx movie add_frame
gfx movie add_frame
# Add the frames twice the slow the movie down
gfx set vis name bladder off
gfx set vis name inner_bladder off
gfx set vis name ureters off
gfx set vis name trigone off

# Read in the deformed node files in the sequence of expansion and add frames
$NUM_DEFORM_FILES=3 # This variable has been set to 3. To view all files set
                    # to 19
$base_name="blad_def";
$group_name_base="deform";
$os=0;
$bladder="bladder"
$ureters="ureters"
$inner_bladder="inner_bladder"

for ($i = 1 ; $i < $NUM_DEFORM_FILES+1; $i++)
{
gfx read node ${base_name}.${i}
gfx read elem ${base_name}.${i}

$os=$i*1000
gfx cre egroup ${ureters}.${i} add_ranges 51+$os..60+$os,69+$os..78+$os from ${group_name_base}.${i}
gfx cre egroup ${bladder}.${i} add_ranges 1+$os..50+$os from ${group_name_base}.${i}
gfx cre egroup ${inner_bladder}.${i} add_ranges 87+$os..136+$os from ${group_name_base}.${i}

# Surfaces
gfx modify g_element ${inner_bladder}.${i} surfaces select_on material inner_bladder_surface face xi3_0 selected_material default_selected render_shaded
gfx modify g_element ${bladder}.${i} surfaces select_on material bladder_surface face xi3_0 selected_material default_selected render_shaded
gfx modify g_element ${ureters}.${i} surfaces select_on material tube selected_material default_selected render_shaded

gfx set order object ${inner_bladder}.${i} position 1
gfx set order object ${ureters}.${i} position 2
gfx set order object ${bladder}.${i} position 3

gfx mod win 1 set antialias 8

gfx movie add_frame
gfx movie add_frame

gfx set vis name ${bladder}.${i} off
gfx set vis name ${ureters}.${i} off
gfx set vis name ${inner_bladder}.${i} off
gfx set vis name ${group_name_base}.${i} off
}

gfx movie end


